/******************************************************************************
 * Top contributors (to current version):
 *   Hans-Joerg Schurr, Andrew Reynolds, Abdalrhman Mohamed
 *
 * This file is part of the cvc5 project.
 *
 * Copyright (c) 2009-2025 by the authors listed in the file AUTHORS
 * in the top-level source directory and their institutional affiliations.
 * All rights reserved.  See the file COPYING in the top-level source
 * directory for licensing information.
 * ****************************************************************************
 *
 * Implementation of proof rule.
 // */

#include <cvc5/cvc5_proof_rule.h>

#include <iostream>

namespace cvc5 {

const char* toString(ProofRule rule)
{
  switch (rule)
  {
    //================================================= Core rules
    case ProofRule::ASSUME: return "ASSUME";
    case ProofRule::SCOPE: return "SCOPE";
    case ProofRule::SUBS: return "SUBS";
    case ProofRule::MACRO_REWRITE: return "MACRO_REWRITE";
    case ProofRule::EVALUATE: return "EVALUATE";
    case ProofRule::DISTINCT_VALUES: return "DISTINCT_VALUES";
    case ProofRule::ACI_NORM: return "ACI_NORM";
    case ProofRule::ABSORB: return "ABSORB";
    case ProofRule::MACRO_SR_EQ_INTRO: return "MACRO_SR_EQ_INTRO";
    case ProofRule::MACRO_SR_PRED_INTRO: return "MACRO_SR_PRED_INTRO";
    case ProofRule::MACRO_SR_PRED_ELIM: return "MACRO_SR_PRED_ELIM";
    case ProofRule::MACRO_SR_PRED_TRANSFORM: return "MACRO_SR_PRED_TRANSFORM";
    case ProofRule::ENCODE_EQ_INTRO: return "ENCODE_EQ_INTRO";
    case ProofRule::DSL_REWRITE: return "DSL_REWRITE";
    case ProofRule::THEORY_REWRITE: return "THEORY_REWRITE";
    case ProofRule::ITE_EQ: return "ITE_EQ";
    //================================================= Trusted rules
    case ProofRule::TRUST: return "TRUST";
    case ProofRule::TRUST_THEORY_REWRITE: return "TRUST_THEORY_REWRITE";
    case ProofRule::SAT_REFUTATION: return "SAT_REFUTATION";
    case ProofRule::DRAT_REFUTATION: return "DRAT_REFUTATION";
    case ProofRule::SAT_EXTERNAL_PROVE: return "SAT_EXTERNAL_PROVE";
    //================================================= Boolean rules
    case ProofRule::RESOLUTION: return "RESOLUTION";
    case ProofRule::CHAIN_RESOLUTION: return "CHAIN_RESOLUTION";
    case ProofRule::FACTORING: return "FACTORING";
    case ProofRule::REORDERING: return "REORDERING";
    case ProofRule::MACRO_RESOLUTION: return "MACRO_RESOLUTION";
    case ProofRule::MACRO_RESOLUTION_TRUST: return "MACRO_RESOLUTION_TRUST";
    case ProofRule::SPLIT: return "SPLIT";
    case ProofRule::EQ_RESOLVE: return "EQ_RESOLVE";
    case ProofRule::MODUS_PONENS: return "MODUS_PONENS";
    case ProofRule::NOT_NOT_ELIM: return "NOT_NOT_ELIM";
    case ProofRule::CONTRA: return "CONTRA";
    case ProofRule::AND_ELIM: return "AND_ELIM";
    case ProofRule::AND_INTRO: return "AND_INTRO";
    case ProofRule::NOT_OR_ELIM: return "NOT_OR_ELIM";
    case ProofRule::IMPLIES_ELIM: return "IMPLIES_ELIM";
    case ProofRule::NOT_IMPLIES_ELIM1: return "NOT_IMPLIES_ELIM1";
    case ProofRule::NOT_IMPLIES_ELIM2: return "NOT_IMPLIES_ELIM2";
    case ProofRule::EQUIV_ELIM1: return "EQUIV_ELIM1";
    case ProofRule::EQUIV_ELIM2: return "EQUIV_ELIM2";
    case ProofRule::NOT_EQUIV_ELIM1: return "NOT_EQUIV_ELIM1";
    case ProofRule::NOT_EQUIV_ELIM2: return "NOT_EQUIV_ELIM2";
    case ProofRule::XOR_ELIM1: return "XOR_ELIM1";
    case ProofRule::XOR_ELIM2: return "XOR_ELIM2";
    case ProofRule::NOT_XOR_ELIM1: return "NOT_XOR_ELIM1";
    case ProofRule::NOT_XOR_ELIM2: return "NOT_XOR_ELIM2";
    case ProofRule::ITE_ELIM1: return "ITE_ELIM1";
    case ProofRule::ITE_ELIM2: return "ITE_ELIM2";
    case ProofRule::NOT_ITE_ELIM1: return "NOT_ITE_ELIM1";
    case ProofRule::NOT_ITE_ELIM2: return "NOT_ITE_ELIM2";
    case ProofRule::NOT_AND: return "NOT_AND";
    //================================================= CNF rules
    case ProofRule::CNF_AND_POS: return "CNF_AND_POS";
    case ProofRule::CNF_AND_NEG: return "CNF_AND_NEG";
    case ProofRule::CNF_OR_POS: return "CNF_OR_POS";
    case ProofRule::CNF_OR_NEG: return "CNF_OR_NEG";
    case ProofRule::CNF_IMPLIES_POS: return "CNF_IMPLIES_POS";
    case ProofRule::CNF_IMPLIES_NEG1: return "CNF_IMPLIES_NEG1";
    case ProofRule::CNF_IMPLIES_NEG2: return "CNF_IMPLIES_NEG2";
    case ProofRule::CNF_EQUIV_POS1: return "CNF_EQUIV_POS1";
    case ProofRule::CNF_EQUIV_POS2: return "CNF_EQUIV_POS2";
    case ProofRule::CNF_EQUIV_NEG1: return "CNF_EQUIV_NEG1";
    case ProofRule::CNF_EQUIV_NEG2: return "CNF_EQUIV_NEG2";
    case ProofRule::CNF_XOR_POS1: return "CNF_XOR_POS1";
    case ProofRule::CNF_XOR_POS2: return "CNF_XOR_POS2";
    case ProofRule::CNF_XOR_NEG1: return "CNF_XOR_NEG1";
    case ProofRule::CNF_XOR_NEG2: return "CNF_XOR_NEG2";
    case ProofRule::CNF_ITE_POS1: return "CNF_ITE_POS1";
    case ProofRule::CNF_ITE_POS2: return "CNF_ITE_POS2";
    case ProofRule::CNF_ITE_POS3: return "CNF_ITE_POS3";
    case ProofRule::CNF_ITE_NEG1: return "CNF_ITE_NEG1";
    case ProofRule::CNF_ITE_NEG2: return "CNF_ITE_NEG2";
    case ProofRule::CNF_ITE_NEG3: return "CNF_ITE_NEG3";
    //================================================= Equality rules
    case ProofRule::REFL: return "REFL";
    case ProofRule::SYMM: return "SYMM";
    case ProofRule::TRANS: return "TRANS";
    case ProofRule::CONG: return "CONG";
    case ProofRule::NARY_CONG: return "NARY_CONG";
    case ProofRule::TRUE_INTRO: return "TRUE_INTRO";
    case ProofRule::TRUE_ELIM: return "TRUE_ELIM";
    case ProofRule::FALSE_INTRO: return "FALSE_INTRO";
    case ProofRule::FALSE_ELIM: return "FALSE_ELIM";
    case ProofRule::HO_APP_ENCODE: return "HO_APP_ENCODE";
    case ProofRule::HO_CONG: return "HO_CONG";
    //================================================= Array rules
    case ProofRule::ARRAYS_READ_OVER_WRITE: return "ARRAYS_READ_OVER_WRITE";
    case ProofRule::ARRAYS_READ_OVER_WRITE_CONTRA:
      return "ARRAYS_READ_OVER_WRITE_CONTRA";
    case ProofRule::ARRAYS_READ_OVER_WRITE_1: return "ARRAYS_READ_OVER_WRITE_1";
    case ProofRule::ARRAYS_EXT: return "ARRAYS_EXT";
    //================================================= Bit-Vector rules
    case ProofRule::MACRO_BV_BITBLAST: return "MACRO_BV_BITBLAST";
    case ProofRule::BV_BITBLAST_STEP: return "BV_BITBLAST_STEP";
    case ProofRule::BV_EAGER_ATOM: return "BV_EAGER_ATOM";
    case ProofRule::BV_POLY_NORM: return "BV_POLY_NORM";
    case ProofRule::BV_POLY_NORM_EQ: return "BV_POLY_NORM_EQ";
    //================================================= Datatype rules
    case ProofRule::DT_SPLIT: return "DT_SPLIT";
    //================================================= Quantifiers rules
    case ProofRule::SKOLEM_INTRO: return "SKOLEM_INTRO";
    case ProofRule::SKOLEMIZE: return "SKOLEMIZE";
    case ProofRule::INSTANTIATE: return "INSTANTIATE";
    case ProofRule::ALPHA_EQUIV: return "ALPHA_EQUIV";
    case ProofRule::QUANT_VAR_REORDERING: return "QUANT_VAR_REORDERING";
    case ProofRule::EXISTS_STRING_LENGTH: return "EXISTS_STRING_LENGTH";
    //================================================= Sets rules
    case ProofRule::SETS_SINGLETON_INJ: return "SETS_SINGLETON_INJ";
    case ProofRule::SETS_EXT: return "SETS_EXT";
    case ProofRule::SETS_FILTER_DOWN: return "SETS_FILTER_DOWN";
    case ProofRule::SETS_FILTER_UP: return "SETS_FILTER_UP";
    //================================================= String rules
    case ProofRule::CONCAT_EQ: return "CONCAT_EQ";
    case ProofRule::CONCAT_UNIFY: return "CONCAT_UNIFY";
    case ProofRule::CONCAT_SPLIT: return "CONCAT_SPLIT";
    case ProofRule::CONCAT_CSPLIT: return "CONCAT_CSPLIT";
    case ProofRule::CONCAT_LPROP: return "CONCAT_LPROP";
    case ProofRule::CONCAT_CPROP: return "CONCAT_CPROP";
    case ProofRule::STRING_DECOMPOSE: return "STRING_DECOMPOSE";
    case ProofRule::STRING_LENGTH_POS: return "STRING_LENGTH_POS";
    case ProofRule::STRING_LENGTH_NON_EMPTY: return "STRING_LENGTH_NON_EMPTY";
    case ProofRule::STRING_REDUCTION: return "STRING_REDUCTION";
    case ProofRule::STRING_EAGER_REDUCTION: return "STRING_EAGER_REDUCTION";
    case ProofRule::RE_INTER: return "RE_INTER";
    case ProofRule::RE_CONCAT: return "RE_CONCAT";
    case ProofRule::RE_UNFOLD_POS: return "RE_UNFOLD_POS";
    case ProofRule::RE_UNFOLD_NEG: return "RE_UNFOLD_NEG";
    case ProofRule::RE_UNFOLD_NEG_CONCAT_FIXED:
      return "RE_UNFOLD_NEG_CONCAT_FIXED";
    case ProofRule::STRING_CODE_INJ: return "STRING_CODE_INJ";
    case ProofRule::STRING_SEQ_UNIT_INJ: return "STRING_SEQ_UNIT_INJ";
    case ProofRule::STRING_EXT: return "STRING_EXT";
    case ProofRule::MACRO_STRING_INFERENCE: return "MACRO_STRING_INFERENCE";
    //================================================= Arith rules
    case ProofRule::MACRO_ARITH_SCALE_SUM_UB: return "MACRO_ARITH_SCALE_SUM_UB";
    case ProofRule::ARITH_SUM_UB: return "ARITH_SUM_UB";
    case ProofRule::ARITH_TRICHOTOMY: return "ARITH_TRICHOTOMY";
    case ProofRule::INT_TIGHT_LB: return "INT_TIGHT_LB";
    case ProofRule::INT_TIGHT_UB: return "INT_TIGHT_UB";
    case ProofRule::ARITH_MULT_SIGN: return "ARITH_MULT_SIGN";
    case ProofRule::ARITH_MULT_POS: return "ARITH_MULT_POS";
    case ProofRule::ARITH_MULT_NEG: return "ARITH_MULT_NEG";
    case ProofRule::ARITH_MULT_TANGENT: return "ARITH_MULT_TANGENT";
    case ProofRule::ARITH_MULT_ABS_COMPARISON:
      return "ARITH_MULT_ABS_COMPARISON";
    case ProofRule::ARITH_REDUCTION: return "ARITH_REDUCTION";
    case ProofRule::ARITH_POLY_NORM: return "ARITH_POLY_NORM";
    case ProofRule::ARITH_POLY_NORM_REL: return "ARITH_POLY_NORM_REL";
    case ProofRule::ARITH_TRANS_PI: return "ARITH_TRANS_PI";
    case ProofRule::ARITH_TRANS_EXP_NEG: return "ARITH_TRANS_EXP_NEG";
    case ProofRule::ARITH_TRANS_EXP_POSITIVITY:
      return "ARITH_TRANS_EXP_POSITIVITY";
    case ProofRule::ARITH_TRANS_EXP_SUPER_LIN:
      return "ARITH_TRANS_EXP_SUPER_LIN";
    case ProofRule::ARITH_TRANS_EXP_ZERO: return "ARITH_TRANS_EXP_ZERO";
    case ProofRule::ARITH_TRANS_EXP_APPROX_ABOVE_NEG:
      return "ARITH_TRANS_EXP_APPROX_ABOVE_NEG";
    case ProofRule::ARITH_TRANS_EXP_APPROX_ABOVE_POS:
      return "ARITH_TRANS_EXP_APPROX_ABOVE_POS";
    case ProofRule::ARITH_TRANS_EXP_APPROX_BELOW:
      return "ARITH_TRANS_EXP_APPROX_BELOW";
    case ProofRule::ARITH_TRANS_SINE_BOUNDS: return "ARITH_TRANS_SINE_BOUNDS";
    case ProofRule::ARITH_TRANS_SINE_SHIFT: return "ARITH_TRANS_SINE_SHIFT";
    case ProofRule::ARITH_TRANS_SINE_SYMMETRY:
      return "ARITH_TRANS_SINE_SYMMETRY";
    case ProofRule::ARITH_TRANS_SINE_TANGENT_ZERO:
      return "ARITH_TRANS_SINE_TANGENT_ZERO";
    case ProofRule::ARITH_TRANS_SINE_TANGENT_PI:
      return "ARITH_TRANS_SINE_TANGENT_PI";
    case ProofRule::ARITH_TRANS_SINE_APPROX_ABOVE_NEG:
      return "ARITH_TRANS_SINE_APPROX_ABOVE_NEG";
    case ProofRule::ARITH_TRANS_SINE_APPROX_ABOVE_POS:
      return "ARITH_TRANS_SINE_APPROX_ABOVE_POS";
    case ProofRule::ARITH_TRANS_SINE_APPROX_BELOW_NEG:
      return "ARITH_TRANS_SINE_APPROX_BELOW_NEG";
    case ProofRule::ARITH_TRANS_SINE_APPROX_BELOW_POS:
      return "ARITH_TRANS_SINE_APPROX_BELOW_POS";
    //================================================= External rules
    case ProofRule::LFSC_RULE: return "LFSC_RULE";
    case ProofRule::ALETHE_RULE: return "ALETHE_RULE";
    //================================================= Unknown rule
    case ProofRule::UNKNOWN: return "UNKNOWN";
    default: return "?";
  }
}

std::ostream& operator<<(std::ostream& out, ProofRule rule)
{
  out << toString(rule);
  return out;
}

const char* toString(cvc5::ProofRewriteRule rule)
{
  switch (rule)
  {
    case ProofRewriteRule::NONE: return "NONE";
    //================================================= ad-hoc rules
    case ProofRewriteRule::DISTINCT_ELIM: return "distinct-elim";
    case ProofRewriteRule::DISTINCT_CARD_CONFLICT:
      return "distinct-card-conflict";
    case ProofRewriteRule::UBV_TO_INT_ELIM: return "ubv-to-int-elim";
    case ProofRewriteRule::INT_TO_BV_ELIM: return "int-to-bv-elim";
    case ProofRewriteRule::MACRO_BOOL_NNF_NORM: return "macro-bool-nnf-norm";
    case ProofRewriteRule::MACRO_BOOL_BV_INVERT_SOLVE: return "macro-bool-bv-invert-solve";
    case ProofRewriteRule::MACRO_ARITH_INT_EQ_CONFLICT:
      return "macro-arith-int-eq-conflict";
    case ProofRewriteRule::MACRO_ARITH_INT_GEQ_TIGHTEN:
      return "macro-arith-int-geq-tighten";
    case ProofRewriteRule::ARITH_STRING_PRED_ENTAIL:
      return "arith-string-pred-entail";
    case ProofRewriteRule::ARITH_STRING_PRED_SAFE_APPROX:
      return "arith-string-pred-safe-approx";
    case ProofRewriteRule::MACRO_ARITH_STRING_PRED_ENTAIL:
      return "macro-arith-string-pred-entail";
    case ProofRewriteRule::ARITH_POW_ELIM: return "arith-pow-elim";
    case ProofRewriteRule::BETA_REDUCE: return "beta-reduce";
    case ProofRewriteRule::LAMBDA_ELIM: return "lambda-elim";
    case ProofRewriteRule::MACRO_LAMBDA_CAPTURE_AVOID:
      return "macro-lambda-capture-avoid";
    case ProofRewriteRule::ARRAYS_SELECT_CONST: return "arrays-select-const";
    case ProofRewriteRule::MACRO_ARRAYS_NORMALIZE_OP: return "macro-arrays-normalize-op";
    case ProofRewriteRule::MACRO_ARRAYS_NORMALIZE_CONSTANT:
      return "macro-arrays-normalize-constant";
    case ProofRewriteRule::ARRAYS_EQ_RANGE_EXPAND:
      return "arrays-eq-range-expand";
    case ProofRewriteRule::EXISTS_ELIM: return "exists-elim";
    case ProofRewriteRule::QUANT_UNUSED_VARS: return "quant-unused-vars";
    case ProofRewriteRule::MACRO_QUANT_MERGE_PRENEX:
      return "macro-quant-merge-prenex";
    case ProofRewriteRule::QUANT_MERGE_PRENEX: return "quant-merge-prenex";
    case ProofRewriteRule::MACRO_QUANT_PRENEX: return "macro-quant-prenex";
    case ProofRewriteRule::MACRO_QUANT_MINISCOPE:
      return "macro-quant-miniscope";
    case ProofRewriteRule::QUANT_MINISCOPE_AND: return "quant-miniscope-and";
    case ProofRewriteRule::QUANT_MINISCOPE_OR: return "quant-miniscope-or";
    case ProofRewriteRule::QUANT_MINISCOPE_ITE: return "quant-miniscope-ite";
    case ProofRewriteRule::QUANT_DT_SPLIT: return "quant-dt-split";
    case ProofRewriteRule::MACRO_QUANT_DT_VAR_EXPAND:
      return "macro-quant-dt-var-expand";
    case ProofRewriteRule::MACRO_QUANT_PARTITION_CONNECTED_FV:
      return "macro-quant-partition-connected-fv";
    case ProofRewriteRule::MACRO_QUANT_VAR_ELIM_EQ:
      return "macro-quant-var-elim-eq";
    case ProofRewriteRule::MACRO_QUANT_VAR_ELIM_INEQ:
      return "macro-quant-var-elim-ineq";
    case ProofRewriteRule::QUANT_VAR_ELIM_EQ: return "quant-var-elim-eq";
    case ProofRewriteRule::MACRO_QUANT_REWRITE_BODY:
      return "macro-quant-rewrite-body";
    case ProofRewriteRule::DT_INST: return "dt-inst";
    case ProofRewriteRule::DT_COLLAPSE_SELECTOR: return "dt-collapse-selector";
    case ProofRewriteRule::DT_COLLAPSE_TESTER: return "dt-collapse-tester";
    case ProofRewriteRule::DT_COLLAPSE_TESTER_SINGLETON:
      return "dt-collapse-tester-singleton";
    case ProofRewriteRule::MACRO_DT_CONS_EQ: return "macro-dt-cons-eq";
    case ProofRewriteRule::DT_CONS_EQ: return "dt-cons-eq";
    case ProofRewriteRule::DT_CONS_EQ_CLASH: return "dt-cons-eq-clash";
    case ProofRewriteRule::DT_CYCLE: return "dt-cycle";
    case ProofRewriteRule::DT_COLLAPSE_UPDATER: return "dt-collapse-updater";
    case ProofRewriteRule::DT_UPDATER_ELIM: return "dt-updater-elim";
    case ProofRewriteRule::DT_MATCH_ELIM: return "dt-match-elim";
    case ProofRewriteRule::MACRO_BV_EXTRACT_CONCAT:
      return "macro-bv-extract-concat";
    case ProofRewriteRule::MACRO_BV_OR_SIMPLIFY: return "macro-bv-or-simplify";
    case ProofRewriteRule::MACRO_BV_AND_SIMPLIFY:
      return "macro-bv-and-simplify";
    case ProofRewriteRule::MACRO_BV_XOR_SIMPLIFY:
      return "macro-bv-xor-simplify";
    case ProofRewriteRule::MACRO_BV_AND_OR_XOR_CONCAT_PULLUP:
      return "macro-bv-and-or-xor-concat-pullup";
    case ProofRewriteRule::MACRO_BV_MULT_SLT_MULT:
      return "macro-bv-mult-slt-mult";
    case ProofRewriteRule::MACRO_BV_CONCAT_EXTRACT_MERGE:
      return "macro-bv-concat-extract-merge";
    case ProofRewriteRule::MACRO_BV_CONCAT_CONSTANT_MERGE:
      return "macro-bv-concat-constant-merge";
    case ProofRewriteRule::MACRO_BV_EQ_SOLVE: return "macro-bv-eq-solve";
    case ProofRewriteRule::BV_UMULO_ELIM: return "bv-umulo-elim";
    case ProofRewriteRule::BV_SMULO_ELIM: return "bv-smulo-elim";
    case ProofRewriteRule::BV_BITWISE_SLICING: return "bv-bitwise-slicing";
    case ProofRewriteRule::BV_REPEAT_ELIM: return "bv-repeat-elim";
    case ProofRewriteRule::STR_CTN_MULTISET_SUBSET:
      return "str-ctn-multiset-subset";
    case ProofRewriteRule::MACRO_STR_EQ_LEN_UNIFY_PREFIX:
      return "macro-str-eq-len-unify-prefix";
    case ProofRewriteRule::MACRO_STR_EQ_LEN_UNIFY:
      return "macro-str-eq-len-unify";
    case ProofRewriteRule::MACRO_STR_SPLIT_CTN: return "macro-str-split-ctn";
    case ProofRewriteRule::MACRO_STR_STRIP_ENDPOINTS:
      return "macro-str-strip-endpoints";
    case ProofRewriteRule::STR_OVERLAP_SPLIT_CTN:
      return "str-overlap-split-ctn";
    case ProofRewriteRule::STR_OVERLAP_ENDPOINTS_CTN:
      return "str-overlap-endpoints-ctn";
    case ProofRewriteRule::STR_OVERLAP_ENDPOINTS_INDEXOF:
      return "str-overlap-endpoints-indexof";
    case ProofRewriteRule::STR_OVERLAP_ENDPOINTS_REPLACE:
      return "str-overlap-endpoints-replace";
    case ProofRewriteRule::MACRO_STR_COMPONENT_CTN:
      return "macro-str-component-ctn";
    case ProofRewriteRule::MACRO_STR_CONST_NCTN_CONCAT:
      return "macro-str-const-nctn-concat";
    case ProofRewriteRule::MACRO_STR_IN_RE_INCLUSION:
      return "macro-str-in-re-inclusion";
    case ProofRewriteRule::MACRO_RE_INTER_UNION_CONST_ELIM:
      return "macro-re-inter-union-const-elim";
    case ProofRewriteRule::SEQ_EVAL_OP: return "seq-eval-op";
    case ProofRewriteRule::STR_INDEXOF_RE_EVAL: return "str-indexof-re-eval";
    case ProofRewriteRule::STR_REPLACE_RE_EVAL: return "str-replace-re-eval";
    case ProofRewriteRule::STR_REPLACE_RE_ALL_EVAL:
      return "str-replace-re-all-eval";
    case ProofRewriteRule::RE_LOOP_ELIM: return "re-loop-elim";
    case ProofRewriteRule::MACRO_RE_INTER_UNION_INCLUSION:
      return "macro-re-inter-union-inclusion";
    case ProofRewriteRule::RE_INTER_INCLUSION: return "re-inter-inclusion";
    case ProofRewriteRule::RE_UNION_INCLUSION: return "re-union-inclusion";
    case ProofRewriteRule::STR_IN_RE_EVAL: return "str-in-re-eval";
    case ProofRewriteRule::STR_IN_RE_CONSUME: return "str-in-re-consume";
    case ProofRewriteRule::STR_IN_RE_CONCAT_STAR_CHAR:
      return "str-in-re-concat-star-char";
    case ProofRewriteRule::STR_IN_RE_SIGMA: return "str-in-re-sigma";
    case ProofRewriteRule::STR_IN_RE_SIGMA_STAR: return "str-in-re-sigma-star";
    case ProofRewriteRule::MACRO_SUBSTR_STRIP_SYM_LENGTH:
      return "macro-substr-strip-sym-length";
    case ProofRewriteRule::SETS_EVAL_OP: return "sets-eval-op";
    case ProofRewriteRule::SETS_INSERT_ELIM:
      return "sets-insert-elim";
      //================================================= RARE rules
            case ProofRewriteRule::ARITH_DIV_TOTAL_ZERO_REAL: return "arith-div-total-zero-real";
case ProofRewriteRule::ARITH_DIV_TOTAL_ZERO_INT: return "arith-div-total-zero-int";
case ProofRewriteRule::ARITH_INT_DIV_TOTAL: return "arith-int-div-total";
case ProofRewriteRule::ARITH_INT_DIV_TOTAL_ONE: return "arith-int-div-total-one";
case ProofRewriteRule::ARITH_INT_DIV_TOTAL_ZERO: return "arith-int-div-total-zero";
case ProofRewriteRule::ARITH_INT_DIV_TOTAL_NEG: return "arith-int-div-total-neg";
case ProofRewriteRule::ARITH_INT_MOD_TOTAL: return "arith-int-mod-total";
case ProofRewriteRule::ARITH_INT_MOD_TOTAL_ONE: return "arith-int-mod-total-one";
case ProofRewriteRule::ARITH_INT_MOD_TOTAL_ZERO: return "arith-int-mod-total-zero";
case ProofRewriteRule::ARITH_INT_MOD_TOTAL_NEG: return "arith-int-mod-total-neg";
case ProofRewriteRule::ARITH_ELIM_GT: return "arith-elim-gt";
case ProofRewriteRule::ARITH_ELIM_LT: return "arith-elim-lt";
case ProofRewriteRule::ARITH_ELIM_INT_GT: return "arith-elim-int-gt";
case ProofRewriteRule::ARITH_ELIM_INT_LT: return "arith-elim-int-lt";
case ProofRewriteRule::ARITH_ELIM_LEQ: return "arith-elim-leq";
case ProofRewriteRule::ARITH_LEQ_NORM: return "arith-leq-norm";
case ProofRewriteRule::ARITH_GEQ_TIGHTEN: return "arith-geq-tighten";
case ProofRewriteRule::ARITH_GEQ_NORM1_INT: return "arith-geq-norm1-int";
case ProofRewriteRule::ARITH_GEQ_NORM1_REAL: return "arith-geq-norm1-real";
case ProofRewriteRule::ARITH_EQ_ELIM_REAL: return "arith-eq-elim-real";
case ProofRewriteRule::ARITH_EQ_ELIM_INT: return "arith-eq-elim-int";
case ProofRewriteRule::ARITH_TO_INT_ELIM: return "arith-to-int-elim";
case ProofRewriteRule::ARITH_TO_INT_ELIM_TO_REAL: return "arith-to-int-elim-to-real";
case ProofRewriteRule::ARITH_DIV_ELIM_TO_REAL1: return "arith-div-elim-to-real1";
case ProofRewriteRule::ARITH_DIV_ELIM_TO_REAL2: return "arith-div-elim-to-real2";
case ProofRewriteRule::ARITH_MOD_OVER_MOD: return "arith-mod-over-mod";
case ProofRewriteRule::ARITH_INT_EQ_CONFLICT: return "arith-int-eq-conflict";
case ProofRewriteRule::ARITH_INT_GEQ_TIGHTEN: return "arith-int-geq-tighten";
case ProofRewriteRule::ARITH_DIVISIBLE_ELIM: return "arith-divisible-elim";
case ProofRewriteRule::ARITH_ABS_EQ: return "arith-abs-eq";
case ProofRewriteRule::ARITH_ABS_INT_GT: return "arith-abs-int-gt";
case ProofRewriteRule::ARITH_ABS_REAL_GT: return "arith-abs-real-gt";
case ProofRewriteRule::ARITH_GEQ_ITE_LIFT: return "arith-geq-ite-lift";
case ProofRewriteRule::ARITH_LEQ_ITE_LIFT: return "arith-leq-ite-lift";
case ProofRewriteRule::ARITH_MIN_LT1: return "arith-min-lt1";
case ProofRewriteRule::ARITH_MIN_LT2: return "arith-min-lt2";
case ProofRewriteRule::ARITH_MAX_GEQ1: return "arith-max-geq1";
case ProofRewriteRule::ARITH_MAX_GEQ2: return "arith-max-geq2";
case ProofRewriteRule::ARRAY_READ_OVER_WRITE: return "array-read-over-write";
case ProofRewriteRule::ARRAY_READ_OVER_WRITE2: return "array-read-over-write2";
case ProofRewriteRule::ARRAY_STORE_OVERWRITE: return "array-store-overwrite";
case ProofRewriteRule::ARRAY_STORE_SELF: return "array-store-self";
case ProofRewriteRule::ARRAY_READ_OVER_WRITE_SPLIT: return "array-read-over-write-split";
case ProofRewriteRule::ARRAY_STORE_SWAP: return "array-store-swap";
case ProofRewriteRule::BOOL_DOUBLE_NOT_ELIM: return "bool-double-not-elim";
case ProofRewriteRule::BOOL_NOT_TRUE: return "bool-not-true";
case ProofRewriteRule::BOOL_NOT_FALSE: return "bool-not-false";
case ProofRewriteRule::BOOL_EQ_TRUE: return "bool-eq-true";
case ProofRewriteRule::BOOL_EQ_FALSE: return "bool-eq-false";
case ProofRewriteRule::BOOL_EQ_NREFL: return "bool-eq-nrefl";
case ProofRewriteRule::BOOL_IMPL_FALSE1: return "bool-impl-false1";
case ProofRewriteRule::BOOL_IMPL_FALSE2: return "bool-impl-false2";
case ProofRewriteRule::BOOL_IMPL_TRUE1: return "bool-impl-true1";
case ProofRewriteRule::BOOL_IMPL_TRUE2: return "bool-impl-true2";
case ProofRewriteRule::BOOL_IMPL_ELIM: return "bool-impl-elim";
case ProofRewriteRule::BOOL_DUAL_IMPL_EQ: return "bool-dual-impl-eq";
case ProofRewriteRule::BOOL_AND_CONF: return "bool-and-conf";
case ProofRewriteRule::BOOL_AND_CONF2: return "bool-and-conf2";
case ProofRewriteRule::BOOL_OR_TAUT: return "bool-or-taut";
case ProofRewriteRule::BOOL_OR_TAUT2: return "bool-or-taut2";
case ProofRewriteRule::BOOL_OR_DE_MORGAN: return "bool-or-de-morgan";
case ProofRewriteRule::BOOL_IMPLIES_DE_MORGAN: return "bool-implies-de-morgan";
case ProofRewriteRule::BOOL_AND_DE_MORGAN: return "bool-and-de-morgan";
case ProofRewriteRule::BOOL_OR_AND_DISTRIB: return "bool-or-and-distrib";
case ProofRewriteRule::BOOL_IMPLIES_OR_DISTRIB: return "bool-implies-or-distrib";
case ProofRewriteRule::BOOL_XOR_REFL: return "bool-xor-refl";
case ProofRewriteRule::BOOL_XOR_NREFL: return "bool-xor-nrefl";
case ProofRewriteRule::BOOL_XOR_FALSE: return "bool-xor-false";
case ProofRewriteRule::BOOL_XOR_TRUE: return "bool-xor-true";
case ProofRewriteRule::BOOL_XOR_COMM: return "bool-xor-comm";
case ProofRewriteRule::BOOL_XOR_ELIM: return "bool-xor-elim";
case ProofRewriteRule::BOOL_NOT_XOR_ELIM: return "bool-not-xor-elim";
case ProofRewriteRule::BOOL_NOT_EQ_ELIM1: return "bool-not-eq-elim1";
case ProofRewriteRule::BOOL_NOT_EQ_ELIM2: return "bool-not-eq-elim2";
case ProofRewriteRule::ITE_NEG_BRANCH: return "ite-neg-branch";
case ProofRewriteRule::ITE_THEN_TRUE: return "ite-then-true";
case ProofRewriteRule::ITE_ELSE_FALSE: return "ite-else-false";
case ProofRewriteRule::ITE_THEN_FALSE: return "ite-then-false";
case ProofRewriteRule::ITE_ELSE_TRUE: return "ite-else-true";
case ProofRewriteRule::ITE_THEN_LOOKAHEAD_SELF: return "ite-then-lookahead-self";
case ProofRewriteRule::ITE_ELSE_LOOKAHEAD_SELF: return "ite-else-lookahead-self";
case ProofRewriteRule::ITE_THEN_LOOKAHEAD_NOT_SELF: return "ite-then-lookahead-not-self";
case ProofRewriteRule::ITE_ELSE_LOOKAHEAD_NOT_SELF: return "ite-else-lookahead-not-self";
case ProofRewriteRule::ITE_EXPAND: return "ite-expand";
case ProofRewriteRule::BOOL_NOT_ITE_ELIM: return "bool-not-ite-elim";
case ProofRewriteRule::ITE_TRUE_COND: return "ite-true-cond";
case ProofRewriteRule::ITE_FALSE_COND: return "ite-false-cond";
case ProofRewriteRule::ITE_NOT_COND: return "ite-not-cond";
case ProofRewriteRule::ITE_EQ_BRANCH: return "ite-eq-branch";
case ProofRewriteRule::ITE_THEN_LOOKAHEAD: return "ite-then-lookahead";
case ProofRewriteRule::ITE_ELSE_LOOKAHEAD: return "ite-else-lookahead";
case ProofRewriteRule::ITE_THEN_NEG_LOOKAHEAD: return "ite-then-neg-lookahead";
case ProofRewriteRule::ITE_ELSE_NEG_LOOKAHEAD: return "ite-else-neg-lookahead";
case ProofRewriteRule::BV_CONCAT_EXTRACT_MERGE: return "bv-concat-extract-merge";
case ProofRewriteRule::BV_EXTRACT_EXTRACT: return "bv-extract-extract";
case ProofRewriteRule::BV_EXTRACT_WHOLE: return "bv-extract-whole";
case ProofRewriteRule::BV_EXTRACT_CONCAT_1: return "bv-extract-concat-1";
case ProofRewriteRule::BV_EXTRACT_CONCAT_2: return "bv-extract-concat-2";
case ProofRewriteRule::BV_EXTRACT_CONCAT_3: return "bv-extract-concat-3";
case ProofRewriteRule::BV_EXTRACT_CONCAT_4: return "bv-extract-concat-4";
case ProofRewriteRule::BV_EQ_EXTRACT_ELIM1: return "bv-eq-extract-elim1";
case ProofRewriteRule::BV_EQ_EXTRACT_ELIM2: return "bv-eq-extract-elim2";
case ProofRewriteRule::BV_EQ_EXTRACT_ELIM3: return "bv-eq-extract-elim3";
case ProofRewriteRule::BV_EXTRACT_NOT: return "bv-extract-not";
case ProofRewriteRule::BV_EXTRACT_SIGN_EXTEND_1: return "bv-extract-sign-extend-1";
case ProofRewriteRule::BV_EXTRACT_SIGN_EXTEND_2: return "bv-extract-sign-extend-2";
case ProofRewriteRule::BV_EXTRACT_SIGN_EXTEND_3: return "bv-extract-sign-extend-3";
case ProofRewriteRule::BV_NOT_XOR: return "bv-not-xor";
case ProofRewriteRule::BV_AND_SIMPLIFY_1: return "bv-and-simplify-1";
case ProofRewriteRule::BV_AND_SIMPLIFY_2: return "bv-and-simplify-2";
case ProofRewriteRule::BV_OR_SIMPLIFY_1: return "bv-or-simplify-1";
case ProofRewriteRule::BV_OR_SIMPLIFY_2: return "bv-or-simplify-2";
case ProofRewriteRule::BV_XOR_SIMPLIFY_1: return "bv-xor-simplify-1";
case ProofRewriteRule::BV_XOR_SIMPLIFY_2: return "bv-xor-simplify-2";
case ProofRewriteRule::BV_XOR_SIMPLIFY_3: return "bv-xor-simplify-3";
case ProofRewriteRule::BV_ULT_ADD_ONE: return "bv-ult-add-one";
case ProofRewriteRule::BV_MULT_SLT_MULT_1: return "bv-mult-slt-mult-1";
case ProofRewriteRule::BV_MULT_SLT_MULT_2: return "bv-mult-slt-mult-2";
case ProofRewriteRule::BV_COMMUTATIVE_XOR: return "bv-commutative-xor";
case ProofRewriteRule::BV_COMMUTATIVE_COMP: return "bv-commutative-comp";
case ProofRewriteRule::BV_ZERO_EXTEND_ELIMINATE_0: return "bv-zero-extend-eliminate-0";
case ProofRewriteRule::BV_SIGN_EXTEND_ELIMINATE_0: return "bv-sign-extend-eliminate-0";
case ProofRewriteRule::BV_NOT_NEQ: return "bv-not-neq";
case ProofRewriteRule::BV_ULT_ONES: return "bv-ult-ones";
case ProofRewriteRule::BV_CONCAT_MERGE_CONST: return "bv-concat-merge-const";
case ProofRewriteRule::BV_COMMUTATIVE_ADD: return "bv-commutative-add";
case ProofRewriteRule::BV_SUB_ELIMINATE: return "bv-sub-eliminate";
case ProofRewriteRule::BV_ITE_WIDTH_ONE: return "bv-ite-width-one";
case ProofRewriteRule::BV_ITE_WIDTH_ONE_NOT: return "bv-ite-width-one-not";
case ProofRewriteRule::BV_EQ_XOR_SOLVE: return "bv-eq-xor-solve";
case ProofRewriteRule::BV_EQ_NOT_SOLVE: return "bv-eq-not-solve";
case ProofRewriteRule::BV_UGT_ELIMINATE: return "bv-ugt-eliminate";
case ProofRewriteRule::BV_UGE_ELIMINATE: return "bv-uge-eliminate";
case ProofRewriteRule::BV_SGT_ELIMINATE: return "bv-sgt-eliminate";
case ProofRewriteRule::BV_SGE_ELIMINATE: return "bv-sge-eliminate";
case ProofRewriteRule::BV_SLE_ELIMINATE: return "bv-sle-eliminate";
case ProofRewriteRule::BV_REDOR_ELIMINATE: return "bv-redor-eliminate";
case ProofRewriteRule::BV_REDAND_ELIMINATE: return "bv-redand-eliminate";
case ProofRewriteRule::BV_ULE_ELIMINATE: return "bv-ule-eliminate";
case ProofRewriteRule::BV_COMP_ELIMINATE: return "bv-comp-eliminate";
case ProofRewriteRule::BV_ROTATE_LEFT_ELIMINATE_1: return "bv-rotate-left-eliminate-1";
case ProofRewriteRule::BV_ROTATE_LEFT_ELIMINATE_2: return "bv-rotate-left-eliminate-2";
case ProofRewriteRule::BV_ROTATE_RIGHT_ELIMINATE_1: return "bv-rotate-right-eliminate-1";
case ProofRewriteRule::BV_ROTATE_RIGHT_ELIMINATE_2: return "bv-rotate-right-eliminate-2";
case ProofRewriteRule::BV_NAND_ELIMINATE: return "bv-nand-eliminate";
case ProofRewriteRule::BV_NOR_ELIMINATE: return "bv-nor-eliminate";
case ProofRewriteRule::BV_XNOR_ELIMINATE: return "bv-xnor-eliminate";
case ProofRewriteRule::BV_SDIV_ELIMINATE: return "bv-sdiv-eliminate";
case ProofRewriteRule::BV_ZERO_EXTEND_ELIMINATE: return "bv-zero-extend-eliminate";
case ProofRewriteRule::BV_UADDO_ELIMINATE: return "bv-uaddo-eliminate";
case ProofRewriteRule::BV_SADDO_ELIMINATE: return "bv-saddo-eliminate";
case ProofRewriteRule::BV_SDIVO_ELIMINATE: return "bv-sdivo-eliminate";
case ProofRewriteRule::BV_SMOD_ELIMINATE: return "bv-smod-eliminate";
case ProofRewriteRule::BV_SREM_ELIMINATE: return "bv-srem-eliminate";
case ProofRewriteRule::BV_USUBO_ELIMINATE: return "bv-usubo-eliminate";
case ProofRewriteRule::BV_SSUBO_ELIMINATE: return "bv-ssubo-eliminate";
case ProofRewriteRule::BV_NEGO_ELIMINATE: return "bv-nego-eliminate";
case ProofRewriteRule::BV_ITE_EQUAL_CHILDREN: return "bv-ite-equal-children";
case ProofRewriteRule::BV_ITE_CONST_CHILDREN_1: return "bv-ite-const-children-1";
case ProofRewriteRule::BV_ITE_CONST_CHILDREN_2: return "bv-ite-const-children-2";
case ProofRewriteRule::BV_ITE_EQUAL_COND_1: return "bv-ite-equal-cond-1";
case ProofRewriteRule::BV_ITE_EQUAL_COND_2: return "bv-ite-equal-cond-2";
case ProofRewriteRule::BV_ITE_EQUAL_COND_3: return "bv-ite-equal-cond-3";
case ProofRewriteRule::BV_ITE_MERGE_THEN_IF: return "bv-ite-merge-then-if";
case ProofRewriteRule::BV_ITE_MERGE_ELSE_IF: return "bv-ite-merge-else-if";
case ProofRewriteRule::BV_ITE_MERGE_THEN_ELSE: return "bv-ite-merge-then-else";
case ProofRewriteRule::BV_ITE_MERGE_ELSE_ELSE: return "bv-ite-merge-else-else";
case ProofRewriteRule::BV_SHL_BY_CONST_0: return "bv-shl-by-const-0";
case ProofRewriteRule::BV_SHL_BY_CONST_1: return "bv-shl-by-const-1";
case ProofRewriteRule::BV_SHL_BY_CONST_2: return "bv-shl-by-const-2";
case ProofRewriteRule::BV_LSHR_BY_CONST_0: return "bv-lshr-by-const-0";
case ProofRewriteRule::BV_LSHR_BY_CONST_1: return "bv-lshr-by-const-1";
case ProofRewriteRule::BV_LSHR_BY_CONST_2: return "bv-lshr-by-const-2";
case ProofRewriteRule::BV_ASHR_BY_CONST_0: return "bv-ashr-by-const-0";
case ProofRewriteRule::BV_ASHR_BY_CONST_1: return "bv-ashr-by-const-1";
case ProofRewriteRule::BV_ASHR_BY_CONST_2: return "bv-ashr-by-const-2";
case ProofRewriteRule::BV_AND_CONCAT_PULLUP: return "bv-and-concat-pullup";
case ProofRewriteRule::BV_OR_CONCAT_PULLUP: return "bv-or-concat-pullup";
case ProofRewriteRule::BV_XOR_CONCAT_PULLUP: return "bv-xor-concat-pullup";
case ProofRewriteRule::BV_AND_CONCAT_PULLUP2: return "bv-and-concat-pullup2";
case ProofRewriteRule::BV_OR_CONCAT_PULLUP2: return "bv-or-concat-pullup2";
case ProofRewriteRule::BV_XOR_CONCAT_PULLUP2: return "bv-xor-concat-pullup2";
case ProofRewriteRule::BV_AND_CONCAT_PULLUP3: return "bv-and-concat-pullup3";
case ProofRewriteRule::BV_OR_CONCAT_PULLUP3: return "bv-or-concat-pullup3";
case ProofRewriteRule::BV_XOR_CONCAT_PULLUP3: return "bv-xor-concat-pullup3";
case ProofRewriteRule::BV_XOR_DUPLICATE: return "bv-xor-duplicate";
case ProofRewriteRule::BV_XOR_ONES: return "bv-xor-ones";
case ProofRewriteRule::BV_XOR_NOT: return "bv-xor-not";
case ProofRewriteRule::BV_NOT_IDEMP: return "bv-not-idemp";
case ProofRewriteRule::BV_ULT_ZERO_1: return "bv-ult-zero-1";
case ProofRewriteRule::BV_ULT_ZERO_2: return "bv-ult-zero-2";
case ProofRewriteRule::BV_ULT_SELF: return "bv-ult-self";
case ProofRewriteRule::BV_LT_SELF: return "bv-lt-self";
case ProofRewriteRule::BV_ULE_SELF: return "bv-ule-self";
case ProofRewriteRule::BV_ULE_ZERO: return "bv-ule-zero";
case ProofRewriteRule::BV_ZERO_ULE: return "bv-zero-ule";
case ProofRewriteRule::BV_SLE_SELF: return "bv-sle-self";
case ProofRewriteRule::BV_ULE_MAX: return "bv-ule-max";
case ProofRewriteRule::BV_NOT_ULT: return "bv-not-ult";
case ProofRewriteRule::BV_MULT_POW2_1: return "bv-mult-pow2-1";
case ProofRewriteRule::BV_MULT_POW2_2: return "bv-mult-pow2-2";
case ProofRewriteRule::BV_MULT_POW2_2B: return "bv-mult-pow2-2b";
case ProofRewriteRule::BV_EXTRACT_MULT_LEADING_BIT: return "bv-extract-mult-leading-bit";
case ProofRewriteRule::BV_UDIV_POW2_NOT_ONE: return "bv-udiv-pow2-not-one";
case ProofRewriteRule::BV_UDIV_ZERO: return "bv-udiv-zero";
case ProofRewriteRule::BV_UDIV_ONE: return "bv-udiv-one";
case ProofRewriteRule::BV_UREM_POW2_NOT_ONE: return "bv-urem-pow2-not-one";
case ProofRewriteRule::BV_UREM_ONE: return "bv-urem-one";
case ProofRewriteRule::BV_UREM_SELF: return "bv-urem-self";
case ProofRewriteRule::BV_SHL_ZERO: return "bv-shl-zero";
case ProofRewriteRule::BV_LSHR_ZERO: return "bv-lshr-zero";
case ProofRewriteRule::BV_ASHR_ZERO: return "bv-ashr-zero";
case ProofRewriteRule::BV_UGT_UREM: return "bv-ugt-urem";
case ProofRewriteRule::BV_ULT_ONE: return "bv-ult-one";
case ProofRewriteRule::BV_MERGE_SIGN_EXTEND_1: return "bv-merge-sign-extend-1";
case ProofRewriteRule::BV_MERGE_SIGN_EXTEND_2: return "bv-merge-sign-extend-2";
case ProofRewriteRule::BV_SIGN_EXTEND_EQ_CONST_1: return "bv-sign-extend-eq-const-1";
case ProofRewriteRule::BV_SIGN_EXTEND_EQ_CONST_2: return "bv-sign-extend-eq-const-2";
case ProofRewriteRule::BV_ZERO_EXTEND_EQ_CONST_1: return "bv-zero-extend-eq-const-1";
case ProofRewriteRule::BV_ZERO_EXTEND_EQ_CONST_2: return "bv-zero-extend-eq-const-2";
case ProofRewriteRule::BV_ZERO_EXTEND_ULT_CONST_1: return "bv-zero-extend-ult-const-1";
case ProofRewriteRule::BV_ZERO_EXTEND_ULT_CONST_2: return "bv-zero-extend-ult-const-2";
case ProofRewriteRule::BV_SIGN_EXTEND_ULT_CONST_1: return "bv-sign-extend-ult-const-1";
case ProofRewriteRule::BV_SIGN_EXTEND_ULT_CONST_2: return "bv-sign-extend-ult-const-2";
case ProofRewriteRule::BV_SIGN_EXTEND_ULT_CONST_3: return "bv-sign-extend-ult-const-3";
case ProofRewriteRule::BV_SIGN_EXTEND_ULT_CONST_4: return "bv-sign-extend-ult-const-4";
case ProofRewriteRule::SETS_EQ_SINGLETON_EMP: return "sets-eq-singleton-emp";
case ProofRewriteRule::SETS_MEMBER_SINGLETON: return "sets-member-singleton";
case ProofRewriteRule::SETS_MEMBER_EMP: return "sets-member-emp";
case ProofRewriteRule::SETS_SUBSET_ELIM: return "sets-subset-elim";
case ProofRewriteRule::SETS_UNION_COMM: return "sets-union-comm";
case ProofRewriteRule::SETS_INTER_COMM: return "sets-inter-comm";
case ProofRewriteRule::SETS_INTER_EMP1: return "sets-inter-emp1";
case ProofRewriteRule::SETS_INTER_EMP2: return "sets-inter-emp2";
case ProofRewriteRule::SETS_MINUS_EMP1: return "sets-minus-emp1";
case ProofRewriteRule::SETS_MINUS_EMP2: return "sets-minus-emp2";
case ProofRewriteRule::SETS_UNION_EMP1: return "sets-union-emp1";
case ProofRewriteRule::SETS_UNION_EMP2: return "sets-union-emp2";
case ProofRewriteRule::SETS_INTER_MEMBER: return "sets-inter-member";
case ProofRewriteRule::SETS_MINUS_MEMBER: return "sets-minus-member";
case ProofRewriteRule::SETS_UNION_MEMBER: return "sets-union-member";
case ProofRewriteRule::SETS_CHOOSE_SINGLETON: return "sets-choose-singleton";
case ProofRewriteRule::SETS_MINUS_SELF: return "sets-minus-self";
case ProofRewriteRule::SETS_IS_EMPTY_ELIM: return "sets-is-empty-elim";
case ProofRewriteRule::SETS_IS_SINGLETON_ELIM: return "sets-is-singleton-elim";
case ProofRewriteRule::STR_EQ_CTN_FALSE: return "str-eq-ctn-false";
case ProofRewriteRule::STR_EQ_CTN_FULL_FALSE1: return "str-eq-ctn-full-false1";
case ProofRewriteRule::STR_EQ_CTN_FULL_FALSE2: return "str-eq-ctn-full-false2";
case ProofRewriteRule::STR_EQ_LEN_FALSE: return "str-eq-len-false";
case ProofRewriteRule::STR_SUBSTR_EMPTY_STR: return "str-substr-empty-str";
case ProofRewriteRule::STR_SUBSTR_EMPTY_RANGE: return "str-substr-empty-range";
case ProofRewriteRule::STR_SUBSTR_EMPTY_START: return "str-substr-empty-start";
case ProofRewriteRule::STR_SUBSTR_EMPTY_START_NEG: return "str-substr-empty-start-neg";
case ProofRewriteRule::STR_SUBSTR_SUBSTR_START_GEQ_LEN: return "str-substr-substr-start-geq-len";
case ProofRewriteRule::STR_SUBSTR_EQ_EMPTY: return "str-substr-eq-empty";
case ProofRewriteRule::STR_SUBSTR_Z_EQ_EMPTY_LEQ: return "str-substr-z-eq-empty-leq";
case ProofRewriteRule::STR_SUBSTR_EQ_EMPTY_LEQ_LEN: return "str-substr-eq-empty-leq-len";
case ProofRewriteRule::STR_LEN_REPLACE_INV: return "str-len-replace-inv";
case ProofRewriteRule::STR_LEN_REPLACE_ALL_INV: return "str-len-replace-all-inv";
case ProofRewriteRule::STR_LEN_UPDATE_INV: return "str-len-update-inv";
case ProofRewriteRule::STR_UPDATE_IN_FIRST_CONCAT: return "str-update-in-first-concat";
case ProofRewriteRule::STR_LEN_SUBSTR_IN_RANGE: return "str-len-substr-in-range";
case ProofRewriteRule::STR_CONCAT_CLASH: return "str-concat-clash";
case ProofRewriteRule::STR_CONCAT_CLASH_REV: return "str-concat-clash-rev";
case ProofRewriteRule::STR_CONCAT_CLASH2: return "str-concat-clash2";
case ProofRewriteRule::STR_CONCAT_CLASH2_REV: return "str-concat-clash2-rev";
case ProofRewriteRule::STR_CONCAT_UNIFY: return "str-concat-unify";
case ProofRewriteRule::STR_CONCAT_UNIFY_REV: return "str-concat-unify-rev";
case ProofRewriteRule::STR_CONCAT_UNIFY_BASE: return "str-concat-unify-base";
case ProofRewriteRule::STR_CONCAT_UNIFY_BASE_REV: return "str-concat-unify-base-rev";
case ProofRewriteRule::STR_PREFIXOF_ELIM: return "str-prefixof-elim";
case ProofRewriteRule::STR_SUFFIXOF_ELIM: return "str-suffixof-elim";
case ProofRewriteRule::STR_PREFIXOF_EQ: return "str-prefixof-eq";
case ProofRewriteRule::STR_SUFFIXOF_EQ: return "str-suffixof-eq";
case ProofRewriteRule::STR_PREFIXOF_ONE: return "str-prefixof-one";
case ProofRewriteRule::STR_SUFFIXOF_ONE: return "str-suffixof-one";
case ProofRewriteRule::STR_SUBSTR_COMBINE1: return "str-substr-combine1";
case ProofRewriteRule::STR_SUBSTR_COMBINE2: return "str-substr-combine2";
case ProofRewriteRule::STR_SUBSTR_COMBINE3: return "str-substr-combine3";
case ProofRewriteRule::STR_SUBSTR_COMBINE4: return "str-substr-combine4";
case ProofRewriteRule::STR_SUBSTR_CONCAT1: return "str-substr-concat1";
case ProofRewriteRule::STR_SUBSTR_CONCAT2: return "str-substr-concat2";
case ProofRewriteRule::STR_SUBSTR_REPLACE: return "str-substr-replace";
case ProofRewriteRule::STR_SUBSTR_FULL: return "str-substr-full";
case ProofRewriteRule::STR_SUBSTR_FULL_EQ: return "str-substr-full-eq";
case ProofRewriteRule::STR_CONTAINS_REFL: return "str-contains-refl";
case ProofRewriteRule::STR_CONTAINS_CONCAT_FIND: return "str-contains-concat-find";
case ProofRewriteRule::STR_CONTAINS_CONCAT_FIND_CONTRA: return "str-contains-concat-find-contra";
case ProofRewriteRule::STR_CONTAINS_SPLIT_CHAR: return "str-contains-split-char";
case ProofRewriteRule::STR_CONTAINS_LEQ_LEN_EQ: return "str-contains-leq-len-eq";
case ProofRewriteRule::STR_CONTAINS_EMP: return "str-contains-emp";
case ProofRewriteRule::STR_CONTAINS_CHAR: return "str-contains-char";
case ProofRewriteRule::STR_AT_ELIM: return "str-at-elim";
case ProofRewriteRule::STR_REPLACE_SELF: return "str-replace-self";
case ProofRewriteRule::STR_REPLACE_ID: return "str-replace-id";
case ProofRewriteRule::STR_REPLACE_PREFIX: return "str-replace-prefix";
case ProofRewriteRule::STR_REPLACE_NO_CONTAINS: return "str-replace-no-contains";
case ProofRewriteRule::STR_REPLACE_FIND_BASE: return "str-replace-find-base";
case ProofRewriteRule::STR_REPLACE_FIND_FIRST_CONCAT: return "str-replace-find-first-concat";
case ProofRewriteRule::STR_REPLACE_EMPTY: return "str-replace-empty";
case ProofRewriteRule::STR_REPLACE_ONE_PRE: return "str-replace-one-pre";
case ProofRewriteRule::STR_REPLACE_FIND_PRE: return "str-replace-find-pre";
case ProofRewriteRule::STR_REPLACE_ALL_NO_CONTAINS: return "str-replace-all-no-contains";
case ProofRewriteRule::STR_REPLACE_RE_NONE: return "str-replace-re-none";
case ProofRewriteRule::STR_REPLACE_RE_ALL_NONE: return "str-replace-re-all-none";
case ProofRewriteRule::STR_LEN_CONCAT_REC: return "str-len-concat-rec";
case ProofRewriteRule::STR_LEN_EQ_ZERO_CONCAT_REC: return "str-len-eq-zero-concat-rec";
case ProofRewriteRule::STR_LEN_EQ_ZERO_BASE: return "str-len-eq-zero-base";
case ProofRewriteRule::STR_INDEXOF_SELF: return "str-indexof-self";
case ProofRewriteRule::STR_INDEXOF_NO_CONTAINS: return "str-indexof-no-contains";
case ProofRewriteRule::STR_INDEXOF_OOB: return "str-indexof-oob";
case ProofRewriteRule::STR_INDEXOF_OOB2: return "str-indexof-oob2";
case ProofRewriteRule::STR_INDEXOF_CONTAINS_PRE: return "str-indexof-contains-pre";
case ProofRewriteRule::STR_INDEXOF_FIND_EMP: return "str-indexof-find-emp";
case ProofRewriteRule::STR_INDEXOF_EQ_IRR: return "str-indexof-eq-irr";
case ProofRewriteRule::STR_INDEXOF_RE_NONE: return "str-indexof-re-none";
case ProofRewriteRule::STR_INDEXOF_RE_EMP_RE: return "str-indexof-re-emp-re";
case ProofRewriteRule::STR_TO_LOWER_CONCAT: return "str-to-lower-concat";
case ProofRewriteRule::STR_TO_UPPER_CONCAT: return "str-to-upper-concat";
case ProofRewriteRule::STR_TO_LOWER_UPPER: return "str-to-lower-upper";
case ProofRewriteRule::STR_TO_UPPER_LOWER: return "str-to-upper-lower";
case ProofRewriteRule::STR_TO_LOWER_LEN: return "str-to-lower-len";
case ProofRewriteRule::STR_TO_UPPER_LEN: return "str-to-upper-len";
case ProofRewriteRule::STR_TO_LOWER_FROM_INT: return "str-to-lower-from-int";
case ProofRewriteRule::STR_TO_UPPER_FROM_INT: return "str-to-upper-from-int";
case ProofRewriteRule::STR_TO_INT_CONCAT_NEG_ONE: return "str-to-int-concat-neg-one";
case ProofRewriteRule::STR_LEQ_EMPTY: return "str-leq-empty";
case ProofRewriteRule::STR_LEQ_EMPTY_EQ: return "str-leq-empty-eq";
case ProofRewriteRule::STR_LEQ_CONCAT_FALSE: return "str-leq-concat-false";
case ProofRewriteRule::STR_LEQ_CONCAT_TRUE: return "str-leq-concat-true";
case ProofRewriteRule::STR_LEQ_CONCAT_BASE_1: return "str-leq-concat-base-1";
case ProofRewriteRule::STR_LEQ_CONCAT_BASE_2: return "str-leq-concat-base-2";
case ProofRewriteRule::STR_LT_ELIM: return "str-lt-elim";
case ProofRewriteRule::STR_FROM_INT_NO_CTN_NONDIGIT: return "str-from-int-no-ctn-nondigit";
case ProofRewriteRule::STR_SUBSTR_CTN_CONTRA: return "str-substr-ctn-contra";
case ProofRewriteRule::STR_SUBSTR_CTN: return "str-substr-ctn";
case ProofRewriteRule::STR_REPLACE_DUAL_CTN: return "str-replace-dual-ctn";
case ProofRewriteRule::STR_REPLACE_DUAL_CTN_FALSE: return "str-replace-dual-ctn-false";
case ProofRewriteRule::STR_REPLACE_SELF_CTN_SIMP: return "str-replace-self-ctn-simp";
case ProofRewriteRule::STR_REPLACE_EMP_CTN_SRC: return "str-replace-emp-ctn-src";
case ProofRewriteRule::STR_SUBSTR_CHAR_START_EQ_LEN: return "str-substr-char-start-eq-len";
case ProofRewriteRule::STR_CONTAINS_REPL_CHAR: return "str-contains-repl-char";
case ProofRewriteRule::STR_CONTAINS_REPL_SELF_TGT_CHAR: return "str-contains-repl-self-tgt-char";
case ProofRewriteRule::STR_CONTAINS_REPL_SELF: return "str-contains-repl-self";
case ProofRewriteRule::STR_CONTAINS_REPL_TGT: return "str-contains-repl-tgt";
case ProofRewriteRule::STR_REPL_REPL_LEN_ID: return "str-repl-repl-len-id";
case ProofRewriteRule::STR_REPL_REPL_SRC_TGT_NO_CTN: return "str-repl-repl-src-tgt-no-ctn";
case ProofRewriteRule::STR_REPL_REPL_TGT_SELF: return "str-repl-repl-tgt-self";
case ProofRewriteRule::STR_REPL_REPL_TGT_NO_CTN: return "str-repl-repl-tgt-no-ctn";
case ProofRewriteRule::STR_REPL_REPL_SRC_SELF: return "str-repl-repl-src-self";
case ProofRewriteRule::STR_REPL_REPL_SRC_INV_NO_CTN1: return "str-repl-repl-src-inv-no-ctn1";
case ProofRewriteRule::STR_REPL_REPL_SRC_INV_NO_CTN2: return "str-repl-repl-src-inv-no-ctn2";
case ProofRewriteRule::STR_REPL_REPL_SRC_INV_NO_CTN3: return "str-repl-repl-src-inv-no-ctn3";
case ProofRewriteRule::STR_REPL_REPL_DUAL_SELF: return "str-repl-repl-dual-self";
case ProofRewriteRule::STR_REPL_REPL_DUAL_ITE1: return "str-repl-repl-dual-ite1";
case ProofRewriteRule::STR_REPL_REPL_DUAL_ITE2: return "str-repl-repl-dual-ite2";
case ProofRewriteRule::STR_REPL_REPL_LOOKAHEAD_ID_SIMP: return "str-repl-repl-lookahead-id-simp";
case ProofRewriteRule::RE_ALL_ELIM: return "re-all-elim";
case ProofRewriteRule::RE_OPT_ELIM: return "re-opt-elim";
case ProofRewriteRule::RE_DIFF_ELIM: return "re-diff-elim";
case ProofRewriteRule::RE_PLUS_ELIM: return "re-plus-elim";
case ProofRewriteRule::RE_CONCAT_STAR_SWAP: return "re-concat-star-swap";
case ProofRewriteRule::RE_CONCAT_STAR_REPEAT: return "re-concat-star-repeat";
case ProofRewriteRule::RE_CONCAT_STAR_SUBSUME1: return "re-concat-star-subsume1";
case ProofRewriteRule::RE_CONCAT_STAR_SUBSUME2: return "re-concat-star-subsume2";
case ProofRewriteRule::RE_CONCAT_MERGE: return "re-concat-merge";
case ProofRewriteRule::RE_UNION_ALL: return "re-union-all";
case ProofRewriteRule::RE_UNION_CONST_ELIM: return "re-union-const-elim";
case ProofRewriteRule::RE_INTER_ALL: return "re-inter-all";
case ProofRewriteRule::RE_STAR_NONE: return "re-star-none";
case ProofRewriteRule::RE_STAR_EMP: return "re-star-emp";
case ProofRewriteRule::RE_STAR_STAR: return "re-star-star";
case ProofRewriteRule::RE_STAR_UNION_DROP_EMP: return "re-star-union-drop-emp";
case ProofRewriteRule::RE_LOOP_NEG: return "re-loop-neg";
case ProofRewriteRule::RE_INTER_CSTRING: return "re-inter-cstring";
case ProofRewriteRule::RE_INTER_CSTRING_NEG: return "re-inter-cstring-neg";
case ProofRewriteRule::STR_SUBSTR_LEN_INCLUDE: return "str-substr-len-include";
case ProofRewriteRule::STR_SUBSTR_LEN_INCLUDE_PRE: return "str-substr-len-include-pre";
case ProofRewriteRule::STR_SUBSTR_LEN_NORM: return "str-substr-len-norm";
case ProofRewriteRule::SEQ_LEN_REV: return "seq-len-rev";
case ProofRewriteRule::SEQ_REV_REV: return "seq-rev-rev";
case ProofRewriteRule::SEQ_REV_CONCAT: return "seq-rev-concat";
case ProofRewriteRule::STR_EQ_REPL_SELF_EMP: return "str-eq-repl-self-emp";
case ProofRewriteRule::STR_EQ_REPL_NO_CHANGE: return "str-eq-repl-no-change";
case ProofRewriteRule::STR_EQ_REPL_TGT_EQ_LEN: return "str-eq-repl-tgt-eq-len";
case ProofRewriteRule::STR_EQ_REPL_LEN_ONE_EMP_PREFIX: return "str-eq-repl-len-one-emp-prefix";
case ProofRewriteRule::STR_EQ_REPL_EMP_TGT_NEMP: return "str-eq-repl-emp-tgt-nemp";
case ProofRewriteRule::STR_EQ_REPL_NEMP_SRC_EMP: return "str-eq-repl-nemp-src-emp";
case ProofRewriteRule::STR_EQ_REPL_SELF_SRC: return "str-eq-repl-self-src";
case ProofRewriteRule::SEQ_LEN_UNIT: return "seq-len-unit";
case ProofRewriteRule::SEQ_NTH_UNIT: return "seq-nth-unit";
case ProofRewriteRule::SEQ_REV_UNIT: return "seq-rev-unit";
case ProofRewriteRule::SEQ_LEN_EMPTY: return "seq-len-empty";
case ProofRewriteRule::RE_IN_EMPTY: return "re-in-empty";
case ProofRewriteRule::RE_IN_SIGMA: return "re-in-sigma";
case ProofRewriteRule::RE_IN_SIGMA_STAR: return "re-in-sigma-star";
case ProofRewriteRule::RE_IN_CSTRING: return "re-in-cstring";
case ProofRewriteRule::RE_IN_COMP: return "re-in-comp";
case ProofRewriteRule::STR_IN_RE_UNION_ELIM: return "str-in-re-union-elim";
case ProofRewriteRule::STR_IN_RE_INTER_ELIM: return "str-in-re-inter-elim";
case ProofRewriteRule::STR_IN_RE_RANGE_ELIM: return "str-in-re-range-elim";
case ProofRewriteRule::STR_IN_RE_CONTAINS: return "str-in-re-contains";
case ProofRewriteRule::STR_IN_RE_FROM_INT_NEMP_DIG_RANGE: return "str-in-re-from-int-nemp-dig-range";
case ProofRewriteRule::STR_IN_RE_FROM_INT_DIG_RANGE: return "str-in-re-from-int-dig-range";
case ProofRewriteRule::EQ_REFL: return "eq-refl";
case ProofRewriteRule::EQ_SYMM: return "eq-symm";
case ProofRewriteRule::EQ_COND_DEQ: return "eq-cond-deq";
case ProofRewriteRule::EQ_ITE_LIFT: return "eq-ite-lift";
case ProofRewriteRule::DISTINCT_BINARY_ELIM: return "distinct-binary-elim";
case ProofRewriteRule::UF_BV2NAT_INT2BV: return "uf-bv2nat-int2bv";
case ProofRewriteRule::UF_BV2NAT_INT2BV_EXTEND: return "uf-bv2nat-int2bv-extend";
case ProofRewriteRule::UF_BV2NAT_INT2BV_EXTRACT: return "uf-bv2nat-int2bv-extract";
case ProofRewriteRule::UF_INT2BV_BV2NAT: return "uf-int2bv-bv2nat";
case ProofRewriteRule::UF_BV2NAT_GEQ_ELIM: return "uf-bv2nat-geq-elim";
case ProofRewriteRule::UF_INT2BV_BVULT_EQUIV: return "uf-int2bv-bvult-equiv";
case ProofRewriteRule::UF_INT2BV_BVULE_EQUIV: return "uf-int2bv-bvule-equiv";
case ProofRewriteRule::UF_SBV_TO_INT_ELIM: return "uf-sbv-to-int-elim";
case ProofRewriteRule::ARITH_SINE_ZERO: return "arith-sine-zero";
case ProofRewriteRule::ARITH_SINE_PI2: return "arith-sine-pi2";
case ProofRewriteRule::ARITH_COSINE_ELIM: return "arith-cosine-elim";
case ProofRewriteRule::ARITH_TANGENT_ELIM: return "arith-tangent-elim";
case ProofRewriteRule::ARITH_SECENT_ELIM: return "arith-secent-elim";
case ProofRewriteRule::ARITH_COSECENT_ELIM: return "arith-cosecent-elim";
case ProofRewriteRule::ARITH_COTANGENT_ELIM: return "arith-cotangent-elim";
case ProofRewriteRule::ARITH_PI_NOT_INT: return "arith-pi-not-int";
case ProofRewriteRule::SETS_CARD_SINGLETON: return "sets-card-singleton";
case ProofRewriteRule::SETS_CARD_UNION: return "sets-card-union";
case ProofRewriteRule::SETS_CARD_MINUS: return "sets-card-minus";
case ProofRewriteRule::SETS_CARD_EMP: return "sets-card-emp";
                    default : return "?";
  }
}

std::ostream& operator<<(std::ostream& out, ProofRewriteRule rule)
{
  out << toString(rule);
  return out;
}

}  // namespace cvc5

namespace std {

size_t hash<cvc5::ProofRule>::operator()(cvc5::ProofRule rule) const
{
  return static_cast<size_t>(rule);
}

std::string to_string(cvc5::ProofRule rule) { return cvc5::toString(rule); }

size_t hash<cvc5::ProofRewriteRule>::operator()(
    cvc5::ProofRewriteRule rule) const
{
  return static_cast<size_t>(rule);
}

std::string to_string(cvc5::ProofRewriteRule rule)
{
  return cvc5::toString(rule);
}

}  // namespace std
