/******************************************************************************
 * Top contributors (to current version):
 *   Mathias Preiner, Morgan Deters, Aina Niemetz
 *
 * This file is part of the cvc5 project.
 *
 * Copyright (c) 2009-2025 by the authors listed in the file AUTHORS
 * in the top-level source directory and their institutional affiliations.
 * All rights reserved.  See the file COPYING in the top-level source
 * directory for licensing information.
 * ****************************************************************************
 *
 * Template for the Type properties source file.
 */

#include "expr/type_properties.h"

namespace cvc5::internal {
namespace kind {

Node mkGroundTerm(NodeManager* nm, TypeConstant tc)
{
  switch (tc)
  {
    // clang-format off
    case BUILTIN_OPERATOR_TYPE: Unhandled() << tc;
    case SEXPR_TYPE: Unhandled() << tc;
    case BOOLEAN_TYPE: return nm->mkConst(false);
    case REAL_TYPE: return nm->mkConstReal(Rational(0));
    case INTEGER_TYPE: return nm->mkConstInt(Rational(0));
    case ROUNDINGMODE_TYPE: return nm->mkConst<RoundingMode>(RoundingMode());
    case STRING_TYPE: return nm->mkConst(::cvc5::internal::String());
    case REGEXP_TYPE: return nm->mkNode(Kind::REGEXP_NONE);
    case BOUND_VAR_LIST_TYPE: Unhandled() << tc;
    case INST_PATTERN_TYPE: Unhandled() << tc;
    case INST_PATTERN_LIST_TYPE: Unhandled() << tc;

      // clang-format on
    default:
      InternalError() << "No ground term known for type constant: " << tc;
  }
}

Node mkGroundTerm(TypeNode typeNode)
{
  AssertArgument(!typeNode.isNull(), typeNode);
  switch (Kind k = typeNode.getKind())
  {
    case Kind::TYPE_CONSTANT:
      return mkGroundTerm(typeNode.getNodeManager(),
                          typeNode.getConst<TypeConstant>());
      // clang-format off
    case Kind::SORT_TYPE: return ::cvc5::internal::theory::builtin::SortProperties::mkGroundTerm(typeNode);
    case Kind::INSTANTIATED_SORT_TYPE: return ::cvc5::internal::theory::builtin::SortProperties::mkGroundTerm(typeNode);
    case Kind::FUNCTION_TYPE: return ::cvc5::internal::theory::uf::FunctionProperties::mkGroundTerm(typeNode);
    case Kind::BITVECTOR_TYPE: return (*cvc5::internal::theory::TypeEnumerator(typeNode));
    case Kind::FINITE_FIELD_TYPE: return (*cvc5::internal::theory::TypeEnumerator(typeNode));
    case Kind::FLOATINGPOINT_TYPE: return (*cvc5::internal::theory::TypeEnumerator(typeNode));
    case Kind::ARRAY_TYPE: return ::cvc5::internal::theory::arrays::ArraysProperties::mkGroundTerm(typeNode);
    case Kind::DATATYPE_TYPE: return typeNode.getDType().mkGroundTerm(typeNode);
    case Kind::PARAMETRIC_DATATYPE: return typeNode.getDType().mkGroundTerm(typeNode);
    case Kind::TUPLE_TYPE: return typeNode.getDType().mkGroundTerm(typeNode);
    case Kind::NULLABLE_TYPE: return typeNode.getDType().mkGroundTerm(typeNode);
    case Kind::SET_TYPE: return ::cvc5::internal::theory::sets::SetsProperties::mkGroundTerm(typeNode);
    case Kind::BAG_TYPE: return ::cvc5::internal::theory::bags::BagsProperties::mkGroundTerm(typeNode);
    case Kind::SEQUENCE_TYPE: return ::cvc5::internal::theory::strings::SequenceProperties::mkGroundTerm(typeNode);

      // clang-format on
    default:
      InternalError() << "A theory kinds file did not provide a ground term "
                      << "or ground term computer for type:\n"
                      << typeNode << "\nof kind " << k;
  }
}

}  // namespace kind
}  // namespace cvc5::internal
