/******************************************************************************
 * Top contributors (to current version):
 *   Gereon Kremer
 *
 * This file is part of the cvc5 project.
 *
 * Copyright (c) 2009-2025 by the authors listed in the file AUTHORS
 * in the top-level source directory and their institutional affiliations.
 * All rights reserved.  See the file COPYING in the top-level source
 * directory for licensing information.
 * ****************************************************************************
 *
 * Option template for option modules.
 *
 * For each <module>_options.toml configuration file, mkoptions.py
 * expands this template and generates a <module>_options.cpp file.
 */
#include "options/parser_options.h"

#include <iostream>

#include "base/check.h"
#include "options/option_exception.h"

namespace cvc5::internal::options {

// clang-format off
std::ostream& operator<<(std::ostream& os, ParsingMode mode)
{
  switch(mode)
  {
    case ParsingMode::DEFAULT: return os << "default";
    case ParsingMode::STRICT: return os << "strict";
    case ParsingMode::LENIENT: return os << "lenient";
    default: Unreachable();
  }
  return os;
}
ParsingMode stringToParsingMode(const std::string& optarg)
{
  if (optarg == "default") return ParsingMode::DEFAULT;
  else if (optarg == "strict") return ParsingMode::STRICT;
  else if (optarg == "lenient") return ParsingMode::LENIENT;
  else if (optarg == "help")
  {
    std::cerr << R"FOOBAR(
  Parsing modes.
Available modes for --parsing-mode are:
+ default
  Be reasonably tolerant of non-conforming inputs.
+ strict
  Be less tolerant of non-conforming inputs.
+ lenient
  Be more tolerant of non-conforming inputs.
)FOOBAR";
    std::exit(1);
  }
  throw OptionException(std::string("unknown option for --parsing-mode: `") +
                        optarg + "'.  Try --parsing-mode=help.");
}
// clang-format on

}  // namespace cvc5::internal::options
