/******************************************************************************
 * Top contributors (to current version):
 *   Gereon Kremer, Mathias Preiner, Aina Niemetz
 *
 * This file is part of the cvc5 project.
 *
 * Copyright (c) 2009-2025 by the authors listed in the file AUTHORS
 * in the top-level source directory and their institutional affiliations.
 * All rights reserved.  See the file COPYING in the top-level source
 * directory for licensing information.
 * ****************************************************************************
 *
 * Contains code for handling command-line options.
 *
 * For each <module>_options.toml configuration file, mkoptions.py
 * expands this template and generates a <module>_options.h file.
 */

#include "cvc5_private.h"

#ifndef CVC5__OPTIONS__QUANTIFIERS_H
#define CVC5__OPTIONS__QUANTIFIERS_H

#include "options/options.h"

// clang-format off

// clang-format on

namespace cvc5::internal::options {

namespace quantifiers::longName {
// clang-format off
  static constexpr const char* conflictBasedInst = "cbqi";
    static constexpr const char* cbqiAllConflict = "cbqi-all-conflict";
    static constexpr const char* cbqiMode = "cbqi-mode";
    static constexpr const char* cbqiSkipRd = "cbqi-skip-rd";
    static constexpr const char* cbqiTConstraint = "cbqi-tconstraint";
    static constexpr const char* cbqiVoExp = "cbqi-vo-exp";
    static constexpr const char* cegisSample = "cegis-sample";
    static constexpr const char* cegqi = "cegqi";
    static constexpr const char* cegqiAll = "cegqi-all";
    static constexpr const char* cegqiBv = "cegqi-bv";
    static constexpr const char* cegqiBvIneqMode = "cegqi-bv-ineq";
    static constexpr const char* cegqiBvInterleaveValue = "cegqi-bv-interleave-value";
    static constexpr const char* cegqiBvLinearize = "cegqi-bv-linear";
    static constexpr const char* cegqiBvRmExtract = "cegqi-bv-rm-extract";
    static constexpr const char* cegqiBvSolveNl = "cegqi-bv-solve-nl";
    static constexpr const char* cegqiFullEffort = "cegqi-full";
    static constexpr const char* cegqiUseInfInt = "cegqi-inf-int";
    static constexpr const char* cegqiUseInfReal = "cegqi-inf-real";
    static constexpr const char* cegqiInnermost = "cegqi-innermost";
    static constexpr const char* cegqiMidpoint = "cegqi-midpoint";
    static constexpr const char* cegqiMinBounds = "cegqi-min-bounds";
    static constexpr const char* cegqiMultiInst = "cegqi-multi-inst";
    static constexpr const char* cegqiNestedQE = "cegqi-nested-qe";
    static constexpr const char* cegqiNopt = "cegqi-nopt";
    static constexpr const char* cegqiRoundUpLowerLia = "cegqi-round-up-lia";
    static constexpr const char* condVarSplitQuant = "cond-var-split-quant";
    static constexpr const char* conjectureGen = "conjecture-gen";
    static constexpr const char* conjectureGenGtEnum = "conjecture-gen-gt-enum";
    static constexpr const char* conjectureGenMaxDepth = "conjecture-gen-max-depth";
    static constexpr const char* conjectureGenPerRound = "conjecture-gen-per-round";
    static constexpr const char* consExpandTriggers = "cons-exp-triggers";
    static constexpr const char* dtStcInduction = "dt-stc-ind";
    static constexpr const char* dtVarExpandQuant = "dt-var-exp-quant";
    static constexpr const char* eMatching = "e-matching";
    static constexpr const char* elimTautQuant = "elim-taut-quant";
    static constexpr const char* enumInst = "enum-inst";
    static constexpr const char* enumInstInterleave = "enum-inst-interleave";
    static constexpr const char* enumInstLimit = "enum-inst-limit";
    static constexpr const char* enumInstRd = "enum-inst-rd";
    static constexpr const char* enumInstStratify = "enum-inst-stratify";
    static constexpr const char* enumInstSum = "enum-inst-sum";
    static constexpr const char* extRewriteQuant = "ext-rewrite-quant";
    static constexpr const char* finiteModelFind = "finite-model-find";
    static constexpr const char* fmfBound = "fmf-bound";
    static constexpr const char* fmfBoundBlast = "fmf-bound-blast";
    static constexpr const char* fmfBoundLazy = "fmf-bound-lazy";
    static constexpr const char* fmfFunWellDefined = "fmf-fun";
    static constexpr const char* fmfFunWellDefinedRelevant = "fmf-fun-rlv";
    static constexpr const char* fmfMbqiMode = "fmf-mbqi";
    static constexpr const char* fmfTypeCompletionThresh = "fmf-type-completion-thresh";
    static constexpr const char* fullSaturateQuant = "full-saturate-quant";
    static constexpr const char* fullSygusVerify = "full-sygus-verify";
    static constexpr const char* globalNegate = "global-negate";
    static constexpr const char* hoElim = "ho-elim";
    static constexpr const char* hoElimStoreAx = "ho-elim-store-ax";
    static constexpr const char* hoMatching = "ho-matching";
    static constexpr const char* hoMergeTermDb = "ho-merge-term-db";
    static constexpr const char* ievalMode = "ieval";
    static constexpr const char* incrementTriggers = "increment-triggers";
    static constexpr const char* instMaxLevel = "inst-max-level";
    static constexpr const char* instMaxRounds = "inst-max-rounds";
    static constexpr const char* instNoEntail = "inst-no-entail";
    static constexpr const char* instWhenMode = "inst-when";
    static constexpr const char* instWhenPhase = "inst-when-phase";
    static constexpr const char* intWfInduction = "int-wf-ind";
    static constexpr const char* iteDtTesterSplitQuant = "ite-dtt-split-quant";
    static constexpr const char* iteLiftQuant = "ite-lift-quant";
    static constexpr const char* literalMatchMode = "literal-matching";
    static constexpr const char* macrosQuant = "macros-quant";
    static constexpr const char* macrosQuantMode = "macros-quant-mode";
    static constexpr const char* mbqi = "mbqi";
    static constexpr const char* mbqiEnum = "mbqi-enum";
    static constexpr const char* mbqiEnumExtVarsGrammar = "mbqi-enum-ext-vars-grammar";
    static constexpr const char* mbqiEnumFreeSymsGrammar = "mbqi-enum-free-syms-grammar";
    static constexpr const char* mbqiEnumGlobalSymGrammar = "mbqi-enum-global-syms-grammar";
    static constexpr const char* mbqiInterleave = "mbqi-interleave";
    static constexpr const char* fmfOneInstPerRound = "mbqi-one-inst-per-round";
    static constexpr const char* miniscopeQuant = "miniscope-quant";
    static constexpr const char* miniscopeQuantUser = "miniscope-quant-user";
    static constexpr const char* multiTriggerCache = "multi-trigger-cache";
    static constexpr const char* multiTriggerLinear = "multi-trigger-linear";
    static constexpr const char* multiTriggerPriority = "multi-trigger-priority";
    static constexpr const char* multiTriggerWhenSingle = "multi-trigger-when-single";
    static constexpr const char* oracles = "oracles";
    static constexpr const char* partialTriggers = "partial-triggers";
    static constexpr const char* poolInst = "pool-inst";
    static constexpr const char* preSkolemQuant = "pre-skolem-quant";
    static constexpr const char* preSkolemQuantNested = "pre-skolem-quant-nested";
    static constexpr const char* prenexQuant = "prenex-quant";
    static constexpr const char* prenexQuantUser = "prenex-quant-user";
    static constexpr const char* printInstMode = "print-inst";
    static constexpr const char* printInstFull = "print-inst-full";
    static constexpr const char* purifyTriggers = "purify-triggers";
    static constexpr const char* quantAlphaEquiv = "quant-alpha-equiv";
    static constexpr const char* quantDynamicSplit = "quant-dsplit";
    static constexpr const char* quantFunWellDefined = "quant-fun-wd";
    static constexpr const char* quantInduction = "quant-ind";
    static constexpr const char* quantRepMode = "quant-rep-mode";
    static constexpr const char* registerQuantBodyTerms = "register-quant-body-terms";
    static constexpr const char* relationalTriggers = "relational-triggers";
    static constexpr const char* relevantTriggers = "relevant-triggers";
    static constexpr const char* quantSubCbqi = "sub-cbqi";
    static constexpr const char* quantSubCbqiTimeout = "sub-cbqi-timeout";
    static constexpr const char* sygus = "sygus";
    static constexpr const char* sygusAddConstGrammar = "sygus-add-const-grammar";
    static constexpr const char* sygusArgRelevant = "sygus-arg-relevant";
    static constexpr const char* sygusInvAutoUnfold = "sygus-auto-unfold";
    static constexpr const char* sygusBoolIteReturnConst = "sygus-bool-ite-return-const";
    static constexpr const char* sygusCoreConnective = "sygus-core-connective";
    static constexpr const char* sygusConstRepairAbort = "sygus-crepair-abort";
    static constexpr const char* sygusEnumMode = "sygus-enum";
    static constexpr const char* sygusEnumFastNumConsts = "sygus-enum-fast-num-consts";
    static constexpr const char* sygusEnumRandomP = "sygus-enum-random-p";
    static constexpr const char* sygusEvalUnfoldMode = "sygus-eval-unfold";
    static constexpr const char* sygusExprMinerCheckTimeout = "sygus-expr-miner-check-timeout";
    static constexpr const char* sygusFilterSolMode = "sygus-filter-sol";
    static constexpr const char* sygusFilterSolRevSubsume = "sygus-filter-sol-rev";
    static constexpr const char* sygusGrammarConsMode = "sygus-grammar-cons";
    static constexpr const char* sygusGrammarNorm = "sygus-grammar-norm";
    static constexpr const char* sygusGrammarUseDisj = "sygus-grammar-use-disj";
    static constexpr const char* sygusInference = "sygus-inference";
    static constexpr const char* sygusInst = "sygus-inst";
    static constexpr const char* sygusInstMode = "sygus-inst-mode";
    static constexpr const char* sygusInstScope = "sygus-inst-scope";
    static constexpr const char* sygusInstTermSel = "sygus-inst-term-sel";
    static constexpr const char* sygusInvTemplMode = "sygus-inv-templ";
    static constexpr const char* sygusInvTemplWhenSyntax = "sygus-inv-templ-when-sg";
    static constexpr const char* sygusMinGrammar = "sygus-min-grammar";
    static constexpr const char* sygusOut = "sygus-out";
    static constexpr const char* sygusUnifPbe = "sygus-pbe";
    static constexpr const char* sygusPbeMultiFair = "sygus-pbe-multi-fair";
    static constexpr const char* sygusPbeMultiFairDiff = "sygus-pbe-multi-fair-diff";
    static constexpr const char* sygusQePreproc = "sygus-qe-preproc";
    static constexpr const char* sygusQueryGen = "sygus-query-gen";
    static constexpr const char* sygusQueryGenDumpFiles = "sygus-query-gen-dump-files";
    static constexpr const char* sygusQueryFilterSolved = "sygus-query-gen-filter-solved";
    static constexpr const char* sygusQueryGenThresh = "sygus-query-gen-thresh";
    static constexpr const char* sygusRecFun = "sygus-rec-fun";
    static constexpr const char* sygusRecFunEvalLimit = "sygus-rec-fun-eval-limit";
    static constexpr const char* sygusRepairConst = "sygus-repair-const";
    static constexpr const char* sygusRepairConstTimeout = "sygus-repair-const-timeout";
    static constexpr const char* sygusRewSynthAccel = "sygus-rr-synth-accel";
    static constexpr const char* sygusRewSynthCheck = "sygus-rr-synth-check";
    static constexpr const char* sygusRewSynthFilterCong = "sygus-rr-synth-filter-cong";
    static constexpr const char* sygusRewSynthFilterMatch = "sygus-rr-synth-filter-match";
    static constexpr const char* sygusRewSynthFilterNonLinear = "sygus-rr-synth-filter-nl";
    static constexpr const char* sygusRewSynthFilterOrder = "sygus-rr-synth-filter-order";
    static constexpr const char* sygusRewSynthInputNVars = "sygus-rr-synth-input-nvars";
    static constexpr const char* sygusRewSynthRec = "sygus-rr-synth-rec";
    static constexpr const char* sygusSampleFpUniform = "sygus-sample-fp-uniform";
    static constexpr const char* sygusSampleGrammar = "sygus-sample-grammar";
    static constexpr const char* sygusSamples = "sygus-samples";
    static constexpr const char* cegqiSingleInvMode = "sygus-si";
    static constexpr const char* cegqiSingleInvAbort = "sygus-si-abort";
    static constexpr const char* cegqiSingleInvReconstruct = "sygus-si-rcons";
    static constexpr const char* cegqiSingleInvReconstructLimit = "sygus-si-rcons-limit";
    static constexpr const char* cegqiSingleInvReconstructP = "sygus-si-rcons-p";
    static constexpr const char* sygusStream = "sygus-stream";
    static constexpr const char* sygusUnifCondIndNoRepeatSol = "sygus-unif-cond-independent-no-repeat-sol";
    static constexpr const char* sygusUnifPi = "sygus-unif-pi";
    static constexpr const char* sygusUnifShuffleCond = "sygus-unif-shuffle-cond";
    static constexpr const char* sygusVerifyInstMaxRounds = "sygus-verify-inst-max-rounds";
    static constexpr const char* sygusVerifyTimeout = "sygus-verify-timeout";
    static constexpr const char* termDbMode = "term-db-mode";
    static constexpr const char* triggerActiveSelMode = "trigger-active-sel";
    static constexpr const char* triggerSelMode = "trigger-sel";
    static constexpr const char* userPatternsQuant = "user-pat";
    static constexpr const char* userPoolQuant = "user-pool";
    static constexpr const char* varElimQuant = "var-elim-quant";
    static constexpr const char* varEntEqElimQuant = "var-ent-eq-elim-quant";
    static constexpr const char* varIneqElimQuant = "var-ineq-elim-quant";
// clang-format on
}

// clang-format off
enum class QcfMode
{
  CONFLICT_ONLY, PROP_EQ,
  __MAX_VALUE = PROP_EQ
};
std::ostream& operator<<(std::ostream& os, QcfMode mode);
QcfMode stringToQcfMode(const std::string& optarg);

enum class CegisSampleMode
{
  NONE, USE, TRUST,
  __MAX_VALUE = TRUST
};
std::ostream& operator<<(std::ostream& os, CegisSampleMode mode);
CegisSampleMode stringToCegisSampleMode(const std::string& optarg);

enum class CegqiBvIneqMode
{
  EQ_SLACK, EQ_BOUNDARY, KEEP,
  __MAX_VALUE = KEEP
};
std::ostream& operator<<(std::ostream& os, CegqiBvIneqMode mode);
CegqiBvIneqMode stringToCegqiBvIneqMode(const std::string& optarg);

enum class CondVarSplitQuantMode
{
  OFF, ON, AGG,
  __MAX_VALUE = AGG
};
std::ostream& operator<<(std::ostream& os, CondVarSplitQuantMode mode);
CondVarSplitQuantMode stringToCondVarSplitQuantMode(const std::string& optarg);

enum class FmfMbqiMode
{
  NONE, FMC, TRUST,
  __MAX_VALUE = TRUST
};
std::ostream& operator<<(std::ostream& os, FmfMbqiMode mode);
FmfMbqiMode stringToFmfMbqiMode(const std::string& optarg);

enum class IevalMode
{
  OFF, USE, USE_LEARN,
  __MAX_VALUE = USE_LEARN
};
std::ostream& operator<<(std::ostream& os, IevalMode mode);
IevalMode stringToIevalMode(const std::string& optarg);

enum class InstWhenMode
{
  FULL, FULL_DELAY, FULL_LAST_CALL, FULL_DELAY_LAST_CALL, LAST_CALL,
  __MAX_VALUE = LAST_CALL
};
std::ostream& operator<<(std::ostream& os, InstWhenMode mode);
InstWhenMode stringToInstWhenMode(const std::string& optarg);

enum class IteLiftQuantMode
{
  NONE, SIMPLE, ALL,
  __MAX_VALUE = ALL
};
std::ostream& operator<<(std::ostream& os, IteLiftQuantMode mode);
IteLiftQuantMode stringToIteLiftQuantMode(const std::string& optarg);

enum class LiteralMatchMode
{
  NONE, USE, AGG_PREDICATE, AGG,
  __MAX_VALUE = AGG
};
std::ostream& operator<<(std::ostream& os, LiteralMatchMode mode);
LiteralMatchMode stringToLiteralMatchMode(const std::string& optarg);

enum class MacrosQuantMode
{
  ALL, GROUND, GROUND_UF,
  __MAX_VALUE = GROUND_UF
};
std::ostream& operator<<(std::ostream& os, MacrosQuantMode mode);
MacrosQuantMode stringToMacrosQuantMode(const std::string& optarg);

enum class MiniscopeQuantMode
{
  OFF, CONJ, FV, CONJ_AND_FV, AGG,
  __MAX_VALUE = AGG
};
std::ostream& operator<<(std::ostream& os, MiniscopeQuantMode mode);
MiniscopeQuantMode stringToMiniscopeQuantMode(const std::string& optarg);

enum class PreSkolemQuantMode
{
  OFF, ON, AGG,
  __MAX_VALUE = AGG
};
std::ostream& operator<<(std::ostream& os, PreSkolemQuantMode mode);
PreSkolemQuantMode stringToPreSkolemQuantMode(const std::string& optarg);

enum class PrenexQuantMode
{
  NONE, SIMPLE, NORMAL,
  __MAX_VALUE = NORMAL
};
std::ostream& operator<<(std::ostream& os, PrenexQuantMode mode);
PrenexQuantMode stringToPrenexQuantMode(const std::string& optarg);

enum class PrintInstMode
{
  LIST, NUM,
  __MAX_VALUE = NUM
};
std::ostream& operator<<(std::ostream& os, PrintInstMode mode);
PrintInstMode stringToPrintInstMode(const std::string& optarg);

enum class QuantDSplitMode
{
  NONE, DEFAULT, AGG,
  __MAX_VALUE = AGG
};
std::ostream& operator<<(std::ostream& os, QuantDSplitMode mode);
QuantDSplitMode stringToQuantDSplitMode(const std::string& optarg);

enum class QuantRepMode
{
  EE, FIRST, DEPTH,
  __MAX_VALUE = DEPTH
};
std::ostream& operator<<(std::ostream& os, QuantRepMode mode);
QuantRepMode stringToQuantRepMode(const std::string& optarg);

enum class SygusEnumMode
{
  SMART, FAST, RANDOM, VAR_AGNOSTIC, AUTO,
  __MAX_VALUE = AUTO
};
std::ostream& operator<<(std::ostream& os, SygusEnumMode mode);
SygusEnumMode stringToSygusEnumMode(const std::string& optarg);

enum class SygusEvalUnfoldMode
{
  NONE, SINGLE, SINGLE_BOOL, MULTI,
  __MAX_VALUE = MULTI
};
std::ostream& operator<<(std::ostream& os, SygusEvalUnfoldMode mode);
SygusEvalUnfoldMode stringToSygusEvalUnfoldMode(const std::string& optarg);

enum class SygusFilterSolMode
{
  NONE, STRONG, WEAK,
  __MAX_VALUE = WEAK
};
std::ostream& operator<<(std::ostream& os, SygusFilterSolMode mode);
SygusFilterSolMode stringToSygusFilterSolMode(const std::string& optarg);

enum class SygusGrammarConsMode
{
  SIMPLE, ANY_CONST, ANY_TERM, ANY_TERM_CONCISE,
  __MAX_VALUE = ANY_TERM_CONCISE
};
std::ostream& operator<<(std::ostream& os, SygusGrammarConsMode mode);
SygusGrammarConsMode stringToSygusGrammarConsMode(const std::string& optarg);

enum class SygusInferenceMode
{
  OFF, TRY, ON,
  __MAX_VALUE = ON
};
std::ostream& operator<<(std::ostream& os, SygusInferenceMode mode);
SygusInferenceMode stringToSygusInferenceMode(const std::string& optarg);

enum class SygusInstMode
{
  PRIORITY_INST, PRIORITY_EVAL, INTERLEAVE,
  __MAX_VALUE = INTERLEAVE
};
std::ostream& operator<<(std::ostream& os, SygusInstMode mode);
SygusInstMode stringToSygusInstMode(const std::string& optarg);

enum class SygusInstScope
{
  IN, OUT, BOTH,
  __MAX_VALUE = BOTH
};
std::ostream& operator<<(std::ostream& os, SygusInstScope mode);
SygusInstScope stringToSygusInstScope(const std::string& optarg);

enum class SygusInstTermSelMode
{
  MIN, MAX, BOTH,
  __MAX_VALUE = BOTH
};
std::ostream& operator<<(std::ostream& os, SygusInstTermSelMode mode);
SygusInstTermSelMode stringToSygusInstTermSelMode(const std::string& optarg);

enum class SygusInvTemplMode
{
  NONE, PRE, POST,
  __MAX_VALUE = POST
};
std::ostream& operator<<(std::ostream& os, SygusInvTemplMode mode);
SygusInvTemplMode stringToSygusInvTemplMode(const std::string& optarg);

enum class SygusSolutionOutMode
{
  STATUS, STATUS_AND_DEF, STANDARD,
  __MAX_VALUE = STANDARD
};
std::ostream& operator<<(std::ostream& os, SygusSolutionOutMode mode);
SygusSolutionOutMode stringToSygusSolutionOutMode(const std::string& optarg);

enum class SygusQueryGenMode
{
  BASIC, SAMPLE_SAT, UNSAT,
  __MAX_VALUE = UNSAT
};
std::ostream& operator<<(std::ostream& os, SygusQueryGenMode mode);
SygusQueryGenMode stringToSygusQueryGenMode(const std::string& optarg);

enum class SygusQueryDumpFilesMode
{
  NONE, ALL, UNSOLVED,
  __MAX_VALUE = UNSOLVED
};
std::ostream& operator<<(std::ostream& os, SygusQueryDumpFilesMode mode);
SygusQueryDumpFilesMode stringToSygusQueryDumpFilesMode(const std::string& optarg);

enum class CegqiSingleInvMode
{
  NONE, USE, ALL,
  __MAX_VALUE = ALL
};
std::ostream& operator<<(std::ostream& os, CegqiSingleInvMode mode);
CegqiSingleInvMode stringToCegqiSingleInvMode(const std::string& optarg);

enum class CegqiSingleInvRconsMode
{
  NONE, TRY, ALL_LIMIT, ALL,
  __MAX_VALUE = ALL
};
std::ostream& operator<<(std::ostream& os, CegqiSingleInvRconsMode mode);
CegqiSingleInvRconsMode stringToCegqiSingleInvRconsMode(const std::string& optarg);

enum class SygusUnifPiMode
{
  NONE, COMPLETE, CENUM, CENUM_IGAIN,
  __MAX_VALUE = CENUM_IGAIN
};
std::ostream& operator<<(std::ostream& os, SygusUnifPiMode mode);
SygusUnifPiMode stringToSygusUnifPiMode(const std::string& optarg);

enum class TermDbMode
{
  ALL, RELEVANT,
  __MAX_VALUE = RELEVANT
};
std::ostream& operator<<(std::ostream& os, TermDbMode mode);
TermDbMode stringToTermDbMode(const std::string& optarg);

enum class TriggerActiveSelMode
{
  ALL, MIN, MAX,
  __MAX_VALUE = MAX
};
std::ostream& operator<<(std::ostream& os, TriggerActiveSelMode mode);
TriggerActiveSelMode stringToTriggerActiveSelMode(const std::string& optarg);

enum class TriggerSelMode
{
  MIN, MAX, MIN_SINGLE_MAX, MIN_SINGLE_ALL, ALL,
  __MAX_VALUE = ALL
};
std::ostream& operator<<(std::ostream& os, TriggerSelMode mode);
TriggerSelMode stringToTriggerSelMode(const std::string& optarg);

enum class UserPatMode
{
  USE, TRUST, STRICT, RESORT, IGNORE, INTERLEAVE,
  __MAX_VALUE = INTERLEAVE
};
std::ostream& operator<<(std::ostream& os, UserPatMode mode);
UserPatMode stringToUserPatMode(const std::string& optarg);

enum class UserPoolMode
{
  USE, TRUST, IGNORE,
  __MAX_VALUE = IGNORE
};
std::ostream& operator<<(std::ostream& os, UserPoolMode mode);
UserPoolMode stringToUserPoolMode(const std::string& optarg);

// clang-format on

#if defined(CVC5_MUZZLED) || defined(CVC5_COMPETITION_MODE)
#  define DO_SEMANTIC_CHECKS_BY_DEFAULT false
#else /* CVC5_MUZZLED || CVC5_COMPETITION_MODE */
#  define DO_SEMANTIC_CHECKS_BY_DEFAULT true
#endif /* CVC5_MUZZLED || CVC5_COMPETITION_MODE */

struct HolderQUANTIFIERS
{
// clang-format off
  bool conflictBasedInst = true;
  bool conflictBasedInstWasSetByUser = false;
  bool cbqiAllConflict = false;
  bool cbqiAllConflictWasSetByUser = false;
  QcfMode cbqiMode = QcfMode::PROP_EQ;
  bool cbqiModeWasSetByUser = false;
  bool cbqiSkipRd = false;
  bool cbqiSkipRdWasSetByUser = false;
  bool cbqiTConstraint = false;
  bool cbqiTConstraintWasSetByUser = false;
  bool cbqiVoExp = false;
  bool cbqiVoExpWasSetByUser = false;
  CegisSampleMode cegisSample = CegisSampleMode::NONE;
  bool cegisSampleWasSetByUser = false;
  bool cegqi = false;
  bool cegqiWasSetByUser = false;
  bool cegqiAll = false;
  bool cegqiAllWasSetByUser = false;
  bool cegqiBv = true;
  bool cegqiBvWasSetByUser = false;
  CegqiBvIneqMode cegqiBvIneqMode = CegqiBvIneqMode::EQ_BOUNDARY;
  bool cegqiBvIneqModeWasSetByUser = false;
  bool cegqiBvInterleaveValue = false;
  bool cegqiBvInterleaveValueWasSetByUser = false;
  bool cegqiBvLinearize = true;
  bool cegqiBvLinearizeWasSetByUser = false;
  bool cegqiBvRmExtract = true;
  bool cegqiBvRmExtractWasSetByUser = false;
  bool cegqiBvSolveNl = false;
  bool cegqiBvSolveNlWasSetByUser = false;
  bool cegqiFullEffort = false;
  bool cegqiFullEffortWasSetByUser = false;
  bool cegqiUseInfInt = false;
  bool cegqiUseInfIntWasSetByUser = false;
  bool cegqiUseInfReal = false;
  bool cegqiUseInfRealWasSetByUser = false;
  bool cegqiInnermost = true;
  bool cegqiInnermostWasSetByUser = false;
  bool cegqiMidpoint = false;
  bool cegqiMidpointWasSetByUser = false;
  bool cegqiMinBounds = false;
  bool cegqiMinBoundsWasSetByUser = false;
  bool cegqiMultiInst = false;
  bool cegqiMultiInstWasSetByUser = false;
  bool cegqiNestedQE = false;
  bool cegqiNestedQEWasSetByUser = false;
  bool cegqiNopt = true;
  bool cegqiNoptWasSetByUser = false;
  bool cegqiRoundUpLowerLia = false;
  bool cegqiRoundUpLowerLiaWasSetByUser = false;
  CondVarSplitQuantMode condVarSplitQuant = CondVarSplitQuantMode::ON;
  bool condVarSplitQuantWasSetByUser = false;
  bool conjectureGen = false;
  bool conjectureGenWasSetByUser = false;
  int64_t conjectureGenGtEnum = 50;
  bool conjectureGenGtEnumWasSetByUser = false;
  int64_t conjectureGenMaxDepth = 3;
  bool conjectureGenMaxDepthWasSetByUser = false;
  int64_t conjectureGenPerRound = 1;
  bool conjectureGenPerRoundWasSetByUser = false;
  bool consExpandTriggers = false;
  bool consExpandTriggersWasSetByUser = false;
  bool dtStcInduction = false;
  bool dtStcInductionWasSetByUser = false;
  bool dtVarExpandQuant = true;
  bool dtVarExpandQuantWasSetByUser = false;
  bool eMatching = true;
  bool eMatchingWasSetByUser = false;
  bool elimTautQuant = true;
  bool elimTautQuantWasSetByUser = false;
  bool enumInst = false;
  bool enumInstWasSetByUser = false;
  bool enumInstInterleave = false;
  bool enumInstInterleaveWasSetByUser = false;
  int64_t enumInstLimit = -1;
  bool enumInstLimitWasSetByUser = false;
  bool enumInstRd = true;
  bool enumInstRdWasSetByUser = false;
  bool enumInstStratify = false;
  bool enumInstStratifyWasSetByUser = false;
  bool enumInstSum = false;
  bool enumInstSumWasSetByUser = false;
  bool extRewriteQuant = false;
  bool extRewriteQuantWasSetByUser = false;
  bool finiteModelFind = false;
  bool finiteModelFindWasSetByUser = false;
  bool fmfBound = false;
  bool fmfBoundWasSetByUser = false;
  bool fmfBoundBlast = false;
  bool fmfBoundBlastWasSetByUser = false;
  bool fmfBoundLazy = false;
  bool fmfBoundLazyWasSetByUser = false;
  bool fmfFunWellDefined = false;
  bool fmfFunWellDefinedWasSetByUser = false;
  bool fmfFunWellDefinedRelevant = false;
  bool fmfFunWellDefinedRelevantWasSetByUser = false;
  FmfMbqiMode fmfMbqiMode = FmfMbqiMode::FMC;
  bool fmfMbqiModeWasSetByUser = false;
  int64_t fmfTypeCompletionThresh = 1000;
  bool fmfTypeCompletionThreshWasSetByUser = false;
  bool fullSaturateQuant = false;
  bool fullSaturateQuantWasSetByUser = false;
  bool fullSygusVerify = false;
  bool fullSygusVerifyWasSetByUser = false;
  bool globalNegate = false;
  bool globalNegateWasSetByUser = false;
  bool hoElim = false;
  bool hoElimWasSetByUser = false;
  bool hoElimStoreAx = true;
  bool hoElimStoreAxWasSetByUser = false;
  bool hoMatching = true;
  bool hoMatchingWasSetByUser = false;
  bool hoMergeTermDb = true;
  bool hoMergeTermDbWasSetByUser = false;
  IevalMode ievalMode = IevalMode::USE;
  bool ievalModeWasSetByUser = false;
  bool incrementTriggers = true;
  bool incrementTriggersWasSetByUser = false;
  int64_t instMaxLevel = -1;
  bool instMaxLevelWasSetByUser = false;
  int64_t instMaxRounds = -1;
  bool instMaxRoundsWasSetByUser = false;
  bool instNoEntail = true;
  bool instNoEntailWasSetByUser = false;
  InstWhenMode instWhenMode = InstWhenMode::FULL_LAST_CALL;
  bool instWhenModeWasSetByUser = false;
  int64_t instWhenPhase = 2;
  bool instWhenPhaseWasSetByUser = false;
  bool intWfInduction = false;
  bool intWfInductionWasSetByUser = false;
  bool iteDtTesterSplitQuant = false;
  bool iteDtTesterSplitQuantWasSetByUser = false;
  IteLiftQuantMode iteLiftQuant = IteLiftQuantMode::SIMPLE;
  bool iteLiftQuantWasSetByUser = false;
  LiteralMatchMode literalMatchMode = LiteralMatchMode::USE;
  bool literalMatchModeWasSetByUser = false;
  bool macrosQuant = false;
  bool macrosQuantWasSetByUser = false;
  MacrosQuantMode macrosQuantMode = MacrosQuantMode::GROUND_UF;
  bool macrosQuantModeWasSetByUser = false;
  bool mbqi = false;
  bool mbqiWasSetByUser = false;
  bool mbqiEnum = false;
  bool mbqiEnumWasSetByUser = false;
  bool mbqiEnumExtVarsGrammar = true;
  bool mbqiEnumExtVarsGrammarWasSetByUser = false;
  bool mbqiEnumFreeSymsGrammar = true;
  bool mbqiEnumFreeSymsGrammarWasSetByUser = false;
  bool mbqiEnumGlobalSymGrammar = false;
  bool mbqiEnumGlobalSymGrammarWasSetByUser = false;
  bool mbqiInterleave = false;
  bool mbqiInterleaveWasSetByUser = false;
  bool fmfOneInstPerRound = false;
  bool fmfOneInstPerRoundWasSetByUser = false;
  MiniscopeQuantMode miniscopeQuant = MiniscopeQuantMode::CONJ_AND_FV;
  bool miniscopeQuantWasSetByUser = false;
  bool miniscopeQuantUser = false;
  bool miniscopeQuantUserWasSetByUser = false;
  bool multiTriggerCache = false;
  bool multiTriggerCacheWasSetByUser = false;
  bool multiTriggerLinear = true;
  bool multiTriggerLinearWasSetByUser = false;
  bool multiTriggerPriority = false;
  bool multiTriggerPriorityWasSetByUser = false;
  bool multiTriggerWhenSingle = false;
  bool multiTriggerWhenSingleWasSetByUser = false;
  bool oracles = false;
  bool oraclesWasSetByUser = false;
  bool partialTriggers = false;
  bool partialTriggersWasSetByUser = false;
  bool poolInst = true;
  bool poolInstWasSetByUser = false;
  PreSkolemQuantMode preSkolemQuant = PreSkolemQuantMode::OFF;
  bool preSkolemQuantWasSetByUser = false;
  bool preSkolemQuantNested = true;
  bool preSkolemQuantNestedWasSetByUser = false;
  PrenexQuantMode prenexQuant = PrenexQuantMode::SIMPLE;
  bool prenexQuantWasSetByUser = false;
  bool prenexQuantUser = false;
  bool prenexQuantUserWasSetByUser = false;
  PrintInstMode printInstMode = PrintInstMode::LIST;
  bool printInstModeWasSetByUser = false;
  bool printInstFull = true;
  bool printInstFullWasSetByUser = false;
  bool purifyTriggers = false;
  bool purifyTriggersWasSetByUser = false;
  bool quantAlphaEquiv = true;
  bool quantAlphaEquivWasSetByUser = false;
  QuantDSplitMode quantDynamicSplit = QuantDSplitMode::DEFAULT;
  bool quantDynamicSplitWasSetByUser = false;
  bool quantFunWellDefined = false;
  bool quantFunWellDefinedWasSetByUser = false;
  bool quantInduction = false;
  bool quantInductionWasSetByUser = false;
  QuantRepMode quantRepMode = QuantRepMode::FIRST;
  bool quantRepModeWasSetByUser = false;
  bool registerQuantBodyTerms = false;
  bool registerQuantBodyTermsWasSetByUser = false;
  bool relationalTriggers = false;
  bool relationalTriggersWasSetByUser = false;
  bool relevantTriggers = false;
  bool relevantTriggersWasSetByUser = false;
  bool quantSubCbqi = false;
  bool quantSubCbqiWasSetByUser = false;
  uint64_t quantSubCbqiTimeout = 0;
  bool quantSubCbqiTimeoutWasSetByUser = false;
  bool sygus = false;
  bool sygusWasSetByUser = false;
  bool sygusAddConstGrammar = true;
  bool sygusAddConstGrammarWasSetByUser = false;
  bool sygusArgRelevant = false;
  bool sygusArgRelevantWasSetByUser = false;
  bool sygusInvAutoUnfold = true;
  bool sygusInvAutoUnfoldWasSetByUser = false;
  bool sygusBoolIteReturnConst = true;
  bool sygusBoolIteReturnConstWasSetByUser = false;
  bool sygusCoreConnective = true;
  bool sygusCoreConnectiveWasSetByUser = false;
  bool sygusConstRepairAbort = false;
  bool sygusConstRepairAbortWasSetByUser = false;
  SygusEnumMode sygusEnumMode = SygusEnumMode::AUTO;
  bool sygusEnumModeWasSetByUser = false;
  uint64_t sygusEnumFastNumConsts = 5;
  bool sygusEnumFastNumConstsWasSetByUser = false;
  double sygusEnumRandomP = 0.5;
  bool sygusEnumRandomPWasSetByUser = false;
  SygusEvalUnfoldMode sygusEvalUnfoldMode = SygusEvalUnfoldMode::SINGLE_BOOL;
  bool sygusEvalUnfoldModeWasSetByUser = false;
  uint64_t sygusExprMinerCheckTimeout = 0;
  bool sygusExprMinerCheckTimeoutWasSetByUser = false;
  SygusFilterSolMode sygusFilterSolMode = SygusFilterSolMode::NONE;
  bool sygusFilterSolModeWasSetByUser = false;
  bool sygusFilterSolRevSubsume = false;
  bool sygusFilterSolRevSubsumeWasSetByUser = false;
  SygusGrammarConsMode sygusGrammarConsMode = SygusGrammarConsMode::SIMPLE;
  bool sygusGrammarConsModeWasSetByUser = false;
  bool sygusGrammarNorm = false;
  bool sygusGrammarNormWasSetByUser = false;
  bool sygusGrammarUseDisj = true;
  bool sygusGrammarUseDisjWasSetByUser = false;
  SygusInferenceMode sygusInference = SygusInferenceMode::OFF;
  bool sygusInferenceWasSetByUser = false;
  bool sygusInst = false;
  bool sygusInstWasSetByUser = false;
  SygusInstMode sygusInstMode = SygusInstMode::PRIORITY_INST;
  bool sygusInstModeWasSetByUser = false;
  SygusInstScope sygusInstScope = SygusInstScope::IN;
  bool sygusInstScopeWasSetByUser = false;
  SygusInstTermSelMode sygusInstTermSel = SygusInstTermSelMode::MIN;
  bool sygusInstTermSelWasSetByUser = false;
  SygusInvTemplMode sygusInvTemplMode = SygusInvTemplMode::POST;
  bool sygusInvTemplModeWasSetByUser = false;
  bool sygusInvTemplWhenSyntax = false;
  bool sygusInvTemplWhenSyntaxWasSetByUser = false;
  bool sygusMinGrammar = true;
  bool sygusMinGrammarWasSetByUser = false;
  SygusSolutionOutMode sygusOut = SygusSolutionOutMode::STANDARD;
  bool sygusOutWasSetByUser = false;
  bool sygusUnifPbe = true;
  bool sygusUnifPbeWasSetByUser = false;
  bool sygusPbeMultiFair = false;
  bool sygusPbeMultiFairWasSetByUser = false;
  int64_t sygusPbeMultiFairDiff = 0;
  bool sygusPbeMultiFairDiffWasSetByUser = false;
  bool sygusQePreproc = false;
  bool sygusQePreprocWasSetByUser = false;
  SygusQueryGenMode sygusQueryGen = SygusQueryGenMode::BASIC;
  bool sygusQueryGenWasSetByUser = false;
  SygusQueryDumpFilesMode sygusQueryGenDumpFiles = SygusQueryDumpFilesMode::NONE;
  bool sygusQueryGenDumpFilesWasSetByUser = false;
  bool sygusQueryFilterSolved = false;
  bool sygusQueryFilterSolvedWasSetByUser = false;
  uint64_t sygusQueryGenThresh = 5;
  bool sygusQueryGenThreshWasSetByUser = false;
  bool sygusRecFun = true;
  bool sygusRecFunWasSetByUser = false;
  uint64_t sygusRecFunEvalLimit = 1000;
  bool sygusRecFunEvalLimitWasSetByUser = false;
  bool sygusRepairConst = true;
  bool sygusRepairConstWasSetByUser = false;
  uint64_t sygusRepairConstTimeout = 0;
  bool sygusRepairConstTimeoutWasSetByUser = false;
  bool sygusRewSynthAccel = false;
  bool sygusRewSynthAccelWasSetByUser = false;
  bool sygusRewSynthCheck = true;
  bool sygusRewSynthCheckWasSetByUser = false;
  bool sygusRewSynthFilterCong = true;
  bool sygusRewSynthFilterCongWasSetByUser = false;
  bool sygusRewSynthFilterMatch = true;
  bool sygusRewSynthFilterMatchWasSetByUser = false;
  bool sygusRewSynthFilterNonLinear = false;
  bool sygusRewSynthFilterNonLinearWasSetByUser = false;
  bool sygusRewSynthFilterOrder = true;
  bool sygusRewSynthFilterOrderWasSetByUser = false;
  int64_t sygusRewSynthInputNVars = 3;
  bool sygusRewSynthInputNVarsWasSetByUser = false;
  bool sygusRewSynthRec = false;
  bool sygusRewSynthRecWasSetByUser = false;
  bool sygusSampleFpUniform = false;
  bool sygusSampleFpUniformWasSetByUser = false;
  bool sygusSampleGrammar = true;
  bool sygusSampleGrammarWasSetByUser = false;
  int64_t sygusSamples = 1000;
  bool sygusSamplesWasSetByUser = false;
  CegqiSingleInvMode cegqiSingleInvMode = CegqiSingleInvMode::NONE;
  bool cegqiSingleInvModeWasSetByUser = false;
  bool cegqiSingleInvAbort = false;
  bool cegqiSingleInvAbortWasSetByUser = false;
  CegqiSingleInvRconsMode cegqiSingleInvReconstruct = CegqiSingleInvRconsMode::ALL;
  bool cegqiSingleInvReconstructWasSetByUser = false;
  int64_t cegqiSingleInvReconstructLimit = 10000;
  bool cegqiSingleInvReconstructLimitWasSetByUser = false;
  double cegqiSingleInvReconstructP = 0.5;
  bool cegqiSingleInvReconstructPWasSetByUser = false;
  bool sygusStream = false;
  bool sygusStreamWasSetByUser = false;
  bool sygusUnifCondIndNoRepeatSol = true;
  bool sygusUnifCondIndNoRepeatSolWasSetByUser = false;
  SygusUnifPiMode sygusUnifPi = SygusUnifPiMode::NONE;
  bool sygusUnifPiWasSetByUser = false;
  bool sygusUnifShuffleCond = false;
  bool sygusUnifShuffleCondWasSetByUser = false;
  int64_t sygusVerifyInstMaxRounds = 10;
  bool sygusVerifyInstMaxRoundsWasSetByUser = false;
  uint64_t sygusVerifyTimeout = 0;
  bool sygusVerifyTimeoutWasSetByUser = false;
  TermDbMode termDbMode = TermDbMode::RELEVANT;
  bool termDbModeWasSetByUser = false;
  TriggerActiveSelMode triggerActiveSelMode = TriggerActiveSelMode::ALL;
  bool triggerActiveSelModeWasSetByUser = false;
  TriggerSelMode triggerSelMode = TriggerSelMode::MIN;
  bool triggerSelModeWasSetByUser = false;
  UserPatMode userPatternsQuant = UserPatMode::TRUST;
  bool userPatternsQuantWasSetByUser = false;
  UserPoolMode userPoolQuant = UserPoolMode::TRUST;
  bool userPoolQuantWasSetByUser = false;
  bool varElimQuant = true;
  bool varElimQuantWasSetByUser = false;
  bool varEntEqElimQuant = false;
  bool varEntEqElimQuantWasSetByUser = false;
  bool varIneqElimQuant = true;
  bool varIneqElimQuantWasSetByUser = false;
// clang-format on
};

#undef DO_SEMANTIC_CHECKS_BY_DEFAULT


}  // namespace cvc5::internal::options

#endif /* CVC5__OPTIONS__QUANTIFIERS_H */
